@@ -5,6 +5,7 @@
 
 #include "StunMessage.hxx"
 
+#include <rutil/compat.hxx>
 #include <rutil/Timer.hxx>
 #include <rutil/Random.hxx>
 #include <rutil/DataStream.hxx>
@@ -146,6 +147,14 @@ StunMessage::init()
    mHasTurnReservationToken = false;
    mHasTurnConnectStat = false;
    mCntTurnXorPeerAddress = 0;
+   mHasTurnRequestedAddressFamily = false;
+   mHasIcePriority = false;
+   mIcePriority = 0;
+   mHasIceUseCandidate = false;
+   mHasIceControlled = false;
+   mIceControlledTieBreaker = 0;
+   mHasIceControlling = false;
+   mIceControllingTieBreaker = 0;
    mErrorCode.reason = 0;
    mUsername = 0;
    mPassword = 0;
@@ -271,6 +280,35 @@ StunMessage::setTurnData(const char* data, unsigned int len)
    }
 }
 
+void 
+StunMessage::setIcePriority(UInt32 priority)
+{
+   mHasIcePriority = true;
+   mIcePriority = priority;
+}
+
+void 
+StunMessage::setIceUseCandidate()
+{
+   mHasIceUseCandidate = true;
+}
+
+void 
+StunMessage::setIceControlled()
+{
+   mHasIceControlled = true;
+   const resip::Data& tieBreaker = resip::Random::getCryptoRandom(8);
+   memcpy(&mIceControlledTieBreaker, tieBreaker.begin(), sizeof(mIceControlledTieBreaker));
+}
+
+void 
+StunMessage::setIceControlling()
+{
+   mHasIceControlling = true;
+   const resip::Data& tieBreaker = resip::Random::getCryptoRandom(8);
+   memcpy(&mIceControllingTieBreaker, tieBreaker.begin(), sizeof(mIceControllingTieBreaker));
+}
+
 void 
 StunMessage::applyXorToAddress(const StunAtrAddress& in, StunAtrAddress& out)
 {
@@ -299,7 +337,7 @@ StunMessage::setStunAtrAddressFromTuple(StunAtrAddress& address, const StunTuple
    {
       // Note:  addr.ipv6 is stored in network byte order
       address.family = StunMessage::IPv6Family;  
-      memcpy(&address.addr.ipv6, tuple.getAddress().to_v6().to_bytes().c_array(), sizeof(address.addr.ipv6));
+      memcpy(&address.addr.ipv6, tuple.getAddress().to_v6().to_bytes().data(), sizeof(address.addr.ipv6));
    }
    else
    {
@@ -317,7 +355,7 @@ StunMessage::setTupleFromStunAtrAddress(StunTuple& tuple, const StunAtrAddress&
    {
       // Note:  addr.ipv6 is stored in network byte order
       asio::ip::address_v6::bytes_type bytes;
-      memcpy(bytes.c_array(), &address.addr.ipv6, bytes.size());
+      memcpy(bytes.data(), &address.addr.ipv6, bytes.size());
       asio::ip::address_v6 addr(bytes);
       tuple.setAddress(addr);
    }
@@ -417,6 +455,7 @@ StunMessage::stunParseAtrUInt64( char* body, unsigned int hdrLen,  UInt64& resul
    else
    {
       memcpy(&result, body, 8);
+      result = ntoh64(result);
       return true;
    }
 }
@@ -474,7 +513,7 @@ StunMessage::stunParseAtrIntegrity( char* body, unsigned int hdrLen,  StunAtrInt
 bool
 StunMessage::stunParseMessage( char* buf, unsigned int bufLen)
 {
-   DebugLog(<< "Received stun message: " << bufLen << " bytes");
+   StackLog(<< "Received stun message: " << bufLen << " bytes");
 	
    if (sizeof(StunMsgHdr) > bufLen)
    {
@@ -998,7 +1037,7 @@ StunMessage::stunParseMessage( char* buf, unsigned int bufLen)
                   WarningLog(<< "invalid attribute length for DontFragment attribute");
                   return false;
                }
-               StackLog(<< "Turn DontFragement = <exists>");
+               StackLog(<< "Turn Dont Fragement = <exists>");
             }
             else
             {
@@ -1040,6 +1079,75 @@ StunMessage::stunParseMessage( char* buf, unsigned int bufLen)
             }
             break;
 					
+         // ICE attributes
+         case IcePriority:
+            if(!mHasIcePriority)
+            {
+               mHasIcePriority = true;
+               if (stunParseAtrUInt32( body, attrLen, mIcePriority) == false)
+               {
+                  WarningLog(<< "problem parsing ICE priority");
+                  return false;
+               }
+               StackLog(<< "Ice Priority = " << mIcePriority);
+            }
+            else
+            {
+               WarningLog(<< "Duplicate IcePriority in message - ignoring.");
+            }
+            break;
+
+         case IceUseCandidate:
+            if(!mHasIceUseCandidate)
+            {
+               mHasIceUseCandidate = true;
+               if(attrLen != 0)
+               {
+                  WarningLog(<< "invalid attribute length for IceUseCandidate attribute");
+                  return false;
+               }
+               StackLog(<< "Ice UseCandidate = <exists>");
+            }
+            else
+            {
+               WarningLog(<< "Duplicate IceUseCandidate in message - ignoring.");
+            }
+            break;
+
+         case IceControlled:
+            if(!mHasIceControlled)
+            {
+               mHasIceControlled = true;
+               if (stunParseAtrUInt64( body, attrLen, mIceControlledTieBreaker) == false)
+               {
+                  WarningLog(<< "problem parsing ICE controlled");
+                  return false;
+               }
+               StackLog(<< "Ice controlled = " << mIceControlledTieBreaker);
+            }
+            else
+            {
+               WarningLog(<< "Duplicate IceControlled in message - ignoring.");
+            }
+            break;
+
+         case IceControlling:
+            if(!mHasIceControlling)
+            {
+               mHasIceControlling = true;
+               if (stunParseAtrUInt64( body, attrLen, mIceControllingTieBreaker) == false)
+               {
+                  WarningLog(<< "problem parsing ICE controlling");
+                  return false;
+               }
+               StackLog(<< "Ice controlling = " << mIceControllingTieBreaker);
+            }
+            else
+            {
+               WarningLog(<< "Duplicate IceControlling in message - ignoring.");
+            }
+            break;
+					
          default:
             if ( atrType <= 0x7FFF ) 
             {
@@ -1080,7 +1188,7 @@ operator<<( EncodeStream& strm, const StunMessage::StunAtrAddress& addr)
    if(addr.family == StunMessage::IPv6Family)
    {
       asio::ip::address_v6::bytes_type bytes;
-      memcpy(bytes.c_array(), &addr.addr.ipv6, bytes.size());
+      memcpy(bytes.data(), &addr.addr.ipv6, bytes.size());
       asio::ip::address_v6 addrv6(bytes);
 
       strm << "[" << addrv6.to_string() << "]:" << addr.port;
@@ -1191,6 +1299,14 @@ StunMessage::encode32(char* buf, UInt32 data)
    return buf + sizeof(UInt32);
 }
 
+char*
+StunMessage::encode64(char* buf, const UInt64 data)
+{
+   UInt64 ndata = hton64(data);
+   memcpy(buf, reinterpret_cast<void*>(&ndata), sizeof(UInt64));
+   return buf + sizeof(UInt64);
+}
+
 char* 
 StunMessage::encode(char* buf, const char* data, unsigned int length)
 {
@@ -1225,8 +1341,8 @@ StunMessage::encodeAtrUInt64(char* ptr, UInt16 type, UInt64 value)
 {
    ptr = encode16(ptr, type);
    ptr = encode16(ptr, 8);
-   memcpy(ptr, reinterpret_cast<void*>(&value), sizeof(UInt64));
-   return ptr + sizeof(UInt64);
+   ptr = encode64(ptr, value);
+   return ptr;
 }
 
 char*
@@ -1489,6 +1605,32 @@ StunMessage::stunEncodeMessage(char* buf, unsigned int bufLen)
       StackLog(<< "Encoding Turn Connect Stat: " << mTurnConnectStat);
       ptr = encodeAtrUInt32(ptr, TurnConnectStat, mTurnConnectStat);
    }   
+   if (mHasTurnRequestedAddressFamily)
+   {
+      StackLog(<< "Encoding Turn RequestedAddressFamily: " << mTurnRequestedAddressFamily);
+      ptr = encodeAtrUInt32(ptr, TurnRequestedAddressFamily, UInt32(mTurnRequestedAddressFamily << 16));
+   }
+   if (mHasIcePriority)
+   {
+      StackLog(<< "Encoding ICE Priority: " << mIcePriority);
+      ptr = encodeAtrUInt32(ptr, IcePriority, mIcePriority);
+   }
+   if (mHasIceUseCandidate)
+   {
+      StackLog(<< "Encoding ICE UseCandidate: <exists>");
+      ptr = encode16(ptr, IceUseCandidate);
+      ptr = encode16(ptr, 0); // 0 attribute length
+   }
+   if (mHasIceControlled)
+   {
+      StackLog(<< "Encoding ICE Controlled: " << mIceControlledTieBreaker);
+      ptr = encodeAtrUInt64(ptr, IceControlled, mIceControlledTieBreaker);
+   }
+   if (mHasIceControlling)
+   {
+      StackLog(<< "Encoding ICE Controlling: " << mIceControllingTieBreaker);
+      ptr = encodeAtrUInt64(ptr, IceControlling, mIceControllingTieBreaker);
+   }
 
    // Update Length in header now - needed in message integrity calculations
    UInt16 msgSize = ptr - buf - sizeof(StunMsgHdr);
@@ -1538,12 +1680,15 @@ StunMessage::stunEncodeFramedMessage(char* buf, unsigned int bufLen)
 void
 StunMessage::computeHmac(char* hmac, const char* input, int length, const char* key, int sizeKey)
 {
+   // !slg! TODO - use newly added rutil/SHA1.hxx class  - will need to add new method to it to support this
    strncpy(hmac,"hmac-not-implemented",20);
 }
 #else
 void
 StunMessage::computeHmac(char* hmac, const char* input, int length, const char* key, int sizeKey)
 {
+   //StackLog(<< "***computeHmac: input='" << Data(input, length).hex() << "', length=" << length << ", key='" << Data(key, sizeKey).hex() << "', keySize=" << sizeKey);
+
    unsigned int resultSize=20;
    HMAC(EVP_sha1(), 
         key, sizeKey, 
@@ -1570,11 +1715,11 @@ StunMessage::createUsernameAndPassword()
 
    if(mRemoteTuple.getAddress().is_v6())
    {
-      *mUsername = Data(mRemoteTuple.getAddress().to_v6().to_bytes().c_array(), mRemoteTuple.getAddress().to_v6().to_bytes().size()).base64encode() + ":";
+      *mUsername = Data(mRemoteTuple.getAddress().to_v6().to_bytes().data(), mRemoteTuple.getAddress().to_v6().to_bytes().size()).base64encode() + ":";
    }
    else
    {
-      *mUsername = Data(mRemoteTuple.getAddress().to_v4().to_bytes().c_array(), mRemoteTuple.getAddress().to_v4().to_bytes().size()).base64encode() + ":";
+      *mUsername = Data(mRemoteTuple.getAddress().to_v4().to_bytes().data(), mRemoteTuple.getAddress().to_v4().to_bytes().size()).base64encode() + ":";
    }
    unsigned int port = mRemoteTuple.getPort();
    *mUsername += Data((char*)&port, sizeof(unsigned int)).base64encode() + ":"; 
@@ -1622,7 +1767,7 @@ StunMessage::getTupleFromUsername(StunTuple& tuple)
       Data addressPart(Data::Share, mUsername->data(), 24); 
       addressPart = addressPart.base64decode();
       asio::ip::address_v6::bytes_type bytes;
-      memcpy(bytes.c_array(), addressPart.data(), bytes.size());
+      memcpy(bytes.data(), addressPart.data(), bytes.size());
       asio::ip::address_v6 addressv6(bytes);
       tuple.setAddress(addressv6);
 
@@ -1637,7 +1782,7 @@ StunMessage::getTupleFromUsername(StunTuple& tuple)
       Data addressPart(Data::Share, mUsername->data(), 8);  
       addressPart = addressPart.base64decode();
       asio::ip::address_v4::bytes_type bytes;
-      memcpy(bytes.c_array(), addressPart.data(), bytes.size());
+      memcpy(bytes.data(), addressPart.data(), bytes.size());
       asio::ip::address_v4 addressv4(bytes);
       tuple.setAddress(addressv4);
 
@@ -1664,6 +1809,21 @@ StunMessage::calculateHmacKey(Data& hmacKey, const Data& longtermAuthenticationP
    }
 }
 
+void
+StunMessage::calculateHmacKeyForHa1(Data& hmacKey, const Data& ha1)
+{
+   assert(mHasUsername);
+
+   if(mHasRealm)  // Longterm authenicationmode
+   {
+      hmacKey = ha1;
+   }
+   else
+   {
+      generateShortTermPasswordForUsername(hmacKey);
+   }
+}
+
 void
 StunMessage::calculateHmacKey(Data& hmacKey, const Data& username, const Data& realm, const Data& longtermAuthenticationPassword)
 {
@@ -1671,7 +1831,7 @@ StunMessage::calculateHmacKey(Data& hmacKey, const Data& username, const Data& r
    r << username << ":" << realm << ":" << longtermAuthenticationPassword;
    hmacKey = r.getBin();
   
-   DebugLog(<< "calculateHmacKey: '" << username << ":" << realm << ":" << longtermAuthenticationPassword << "' = '" << hmacKey.hex() << "'");
+   StackLog(<< "calculateHmacKey: '" << username << ":" << realm << ":" << longtermAuthenticationPassword << "' = '" << hmacKey.hex() << "'");
 }
 
 bool 
