@@ -11,16 +11,8 @@
 using namespace std;
 using namespace resip;
 
-#define UDP_RT0 100  // RTO - Estimate of Roundtrip time - 100ms is recommened for fixed line transport - the initial value should be configurable
-                     // Should also be calculation this on the fly
-#define UDP_MAX_RETRANSMITS    7       // Defined by RFC5389 (Rc) - should be configurable
-#define TCP_RESPONSE_TIME      39500   // Defined by RFC5389 (Ti) - should be configurable
-#define UDP_Rm                 16      // Defined by RFC5389 - should be configurable
-#define UDP_FINAL_REQUEST_TIME (UDP_RT0 * UDP_Rm)  // Defined by RFC5389
-
 //#define TURN_CHANNEL_BINDING_REFRESH_SECONDS 20   // TESTING only
 #define TURN_CHANNEL_BINDING_REFRESH_SECONDS 240   // 4 minuntes - this is one minute before the permission will expire, Note:  ChannelBinding refreshes also refresh permissions
-
 #define SOFTWARE_STRING "reTURN Async Client 0.3 - RFC5389/turn-12   "  // Note padding size to a multiple of 4, to help compatibility with older clients
 
 namespace reTurn {
@@ -65,14 +57,12 @@ TurnAsyncSocket::disableTurnAsyncHandler()
 void
 TurnAsyncSocket::requestSharedSecret()
 {
-   mGuards.push(mAsyncSocketBase.shared_from_this());
-   mIOService.post(boost::bind(&TurnAsyncSocket::doRequestSharedSecret, this));
+   mIOService.dispatch(weak_bind<AsyncSocketBase, void()>( mAsyncSocketBase.shared_from_this(), boost::bind(&TurnAsyncSocket::doRequestSharedSecret, this)));
 }
 
 void
 TurnAsyncSocket::doRequestSharedSecret()
 {
-   GuardReleaser guardReleaser(mGuards);
    // Should we check here if TLS and deny?
 
    // Ensure Connected
@@ -93,14 +83,12 @@ TurnAsyncSocket::doRequestSharedSecret()
 void
 TurnAsyncSocket::setUsernameAndPassword(const char* username, const char* password, bool shortTermAuth)
 {
-   mGuards.push(mAsyncSocketBase.shared_from_this());
-   mIOService.post(boost::bind(&TurnAsyncSocket::doSetUsernameAndPassword, this, new Data(username), new Data(password), shortTermAuth));
+   mIOService.dispatch(weak_bind<AsyncSocketBase, void()>( mAsyncSocketBase.shared_from_this(), boost::bind(&TurnAsyncSocket::doSetUsernameAndPassword, this, new Data(username), new Data(password), shortTermAuth)));
 }
 
 void 
 TurnAsyncSocket::doSetUsernameAndPassword(Data* username, Data* password, bool shortTermAuth)
 {
-   GuardReleaser guardReleaser(mGuards);
    mUsername = *username;
    mPassword = *password;
    if(shortTermAuth)
@@ -112,21 +100,32 @@ TurnAsyncSocket::doSetUsernameAndPassword(Data* username, Data* password, bool s
    delete password;
 }
 
+void 
+TurnAsyncSocket::setLocalPassword(const char* password)
+{
+   mIOService.dispatch(weak_bind<AsyncSocketBase, void()>( mAsyncSocketBase.shared_from_this(), boost::bind(&TurnAsyncSocket::doSetLocalPassword, this, new Data(password))));
+}
+
+void 
+TurnAsyncSocket::doSetLocalPassword(Data* password)
+{
+   mLocalHmacKey = *password;
+   delete password;
+}
+
 void 
 TurnAsyncSocket::bindRequest()
 {
-   mGuards.push(mAsyncSocketBase.shared_from_this());
-   mIOService.post(boost::bind(&TurnAsyncSocket::doBindRequest, this));
+   mIOService.dispatch(weak_bind<AsyncSocketBase, void()>(mAsyncSocketBase.shared_from_this(), boost::bind(&TurnAsyncSocket::doBindRequest, this)));
 }
 
 void 
 TurnAsyncSocket::doBindRequest()
 {
-   GuardReleaser guardReleaser(mGuards);
    // Ensure Connected
    if(!mAsyncSocketBase.isConnected())
    {
-      if(mTurnAsyncSocketHandler) mTurnAsyncSocketHandler->onBindFailure(getSocketDescriptor(), asio::error_code(reTurn::NotConnected, asio::error::misc_category));
+      if(mTurnAsyncSocketHandler) mTurnAsyncSocketHandler->onBindFailure(getSocketDescriptor(), asio::error_code(reTurn::NotConnected, asio::error::misc_category), StunTuple());
    }
    else
    {
@@ -136,6 +135,34 @@ TurnAsyncSocket::doBindRequest()
    }
 }
 
+void
+TurnAsyncSocket::connectivityCheck(const StunTuple& targetAddr, UInt32 peerRflxPriority, bool setIceControlling, bool setIceControlled, unsigned int numRetransmits, unsigned int retrans_iterval_ms)
+{
+   assert(setIceControlling || setIceControlled);
+   mIOService.dispatch(weak_bind<AsyncSocketBase, void()>(mAsyncSocketBase.shared_from_this(), boost::bind(&TurnAsyncSocket::doConnectivityCheck, this, new StunTuple(targetAddr.getTransportType(), targetAddr.getAddress(), targetAddr.getPort()), peerRflxPriority, setIceControlling, setIceControlled, numRetransmits, retrans_iterval_ms)));
+}
+
+void
+TurnAsyncSocket::doConnectivityCheck(StunTuple* targetAddr, UInt32 peerRflxPriority, bool setIceControlling, bool setIceControlled, unsigned int numRetransmits, unsigned int retrans_iterval_ms)
+{
+   // Form Stun Bind request
+   StunMessage* request = createNewStunMessage(StunMessage::StunClassRequest, StunMessage::BindMethod);
+   request->setIcePriority(peerRflxPriority);
+   if (setIceControlling)
+   {
+      request->setIceControlling();
+      request->setIceUseCandidate();
+   }
+   else if (setIceControlled)
+   {
+      request->setIceControlled();
+   }
+   request->mHasFingerprint = true;
+
+   sendStunMessage(request, false, numRetransmits, retrans_iterval_ms, targetAddr);
+   delete targetAddr;
+}
+
 void
 TurnAsyncSocket::createAllocation(unsigned int lifetime,
                                   unsigned int bandwidth,
@@ -143,12 +170,7 @@ TurnAsyncSocket::createAllocation(unsigned int lifetime,
                                   UInt64 reservationToken,
                                   StunTuple::TransportType requestedTransportType)
 {
-   mGuards.push(mAsyncSocketBase.shared_from_this());
-   mIOService.post(boost::bind(&TurnAsyncSocket::doCreateAllocation, this, lifetime, 
-                                                                           bandwidth, 
-                                                                           requestedProps, 
-                                                                           reservationToken, 
-                                                                           requestedTransportType));
+   mIOService.dispatch(weak_bind<AsyncSocketBase, void()>( mAsyncSocketBase.shared_from_this(), boost::bind(&TurnAsyncSocket::doCreateAllocation, this, lifetime, bandwidth, requestedProps, reservationToken, requestedTransportType )));
 }
 
 void
@@ -158,8 +180,6 @@ TurnAsyncSocket::doCreateAllocation(unsigned int lifetime,
                                     UInt64 reservationToken,
                                     StunTuple::TransportType requestedTransportType)
 {
-   GuardReleaser guardReleaser(mGuards);
-
    // Store Allocation Properties
    mRequestedTransportType = requestedTransportType;
 
@@ -238,14 +258,12 @@ TurnAsyncSocket::doCreateAllocation(unsigned int lifetime,
 void 
 TurnAsyncSocket::refreshAllocation(unsigned int lifetime)
 {
-   mGuards.push(mAsyncSocketBase.shared_from_this());
-   mIOService.post(boost::bind(&TurnAsyncSocket::doRefreshAllocation, this, lifetime));
+   mIOService.dispatch(weak_bind<AsyncSocketBase, void()>(mAsyncSocketBase.shared_from_this(), boost::bind(&TurnAsyncSocket::doRefreshAllocation, this, lifetime)));
 }
 
 void 
 TurnAsyncSocket::doRefreshAllocation(unsigned int lifetime)
 {
-   GuardReleaser guardReleaser(mGuards);
    if(!mHaveAllocation)
    {
       if(mTurnAsyncSocketHandler) mTurnAsyncSocketHandler->onRefreshFailure(getSocketDescriptor(), asio::error_code(NoAllocation, asio::error::misc_category));
@@ -276,8 +294,7 @@ TurnAsyncSocket::doRefreshAllocation(unsigned int lifetime)
 void 
 TurnAsyncSocket::destroyAllocation()
 {
-   mGuards.push(mAsyncSocketBase.shared_from_this());
-   mIOService.post(boost::bind(&TurnAsyncSocket::doDestroyAllocation, this));
+   mIOService.dispatch(weak_bind<AsyncSocketBase, void()>( mAsyncSocketBase.shared_from_this(), boost::bind( &TurnAsyncSocket::doDestroyAllocation, this )));
 }
 
 void 
@@ -289,15 +306,12 @@ TurnAsyncSocket::doDestroyAllocation()
 void
 TurnAsyncSocket::setActiveDestination(const asio::ip::address& address, unsigned short port)
 {
-   mGuards.push(mAsyncSocketBase.shared_from_this());
-   mIOService.post(boost::bind(&TurnAsyncSocket::doSetActiveDestination, this, address, port));
+   mIOService.dispatch(weak_bind<AsyncSocketBase, void()>( mAsyncSocketBase.shared_from_this(), boost::bind( &TurnAsyncSocket::doSetActiveDestination, this, address, port )));
 }
 
 void
 TurnAsyncSocket::doSetActiveDestination(const asio::ip::address& address, unsigned short port)
 {
-   GuardReleaser guardReleaser(mGuards);
-
    // Setup Remote Peer 
    StunTuple remoteTuple(mRelayTransportType, address, port);
    RemotePeer* remotePeer = mChannelManager.findRemotePeerByPeerAddress(remoteTuple);
@@ -312,7 +326,7 @@ TurnAsyncSocket::doSetActiveDestination(const asio::ip::address& address, unsign
       assert(mActiveDestination);
       doChannelBinding(*mActiveDestination);
    }
-   DebugLog(<< "TurnAsyncSocket::doSetActiveDestination: Active Destination set to: " << remoteTuple << ", channel=" << mActiveDestination->getChannel());
+   DebugLog(<< "TurnAsyncSocket::doSetActiveDestination: Active Destination set to: " << remoteTuple);
    if(mTurnAsyncSocketHandler) mTurnAsyncSocketHandler->onSetActiveDestinationSuccess(getSocketDescriptor());
 }
 
@@ -335,20 +349,19 @@ void TurnAsyncSocket::doChannelBinding(RemotePeer& remotePeer)
    {
       remotePeer.setChannelConfirmed();
    }
+
+   if(mTurnAsyncSocketHandler) mTurnAsyncSocketHandler->onChannelBindRequestSent(getSocketDescriptor(), remotePeer.getChannel());
 }
 
 void
 TurnAsyncSocket::clearActiveDestination()
 {
-   mGuards.push(mAsyncSocketBase.shared_from_this());
-   mIOService.post(boost::bind(&TurnAsyncSocket::doClearActiveDestination, this));
+   mIOService.dispatch(weak_bind<AsyncSocketBase, void()>( mAsyncSocketBase.shared_from_this(), boost::bind( &TurnAsyncSocket::doClearActiveDestination, this )));
 }
 
 void
 TurnAsyncSocket::doClearActiveDestination()
 {
-   GuardReleaser guardReleaser(mGuards);
-
    // ensure there is an allocation
    if(!mHaveAllocation)
    {
@@ -387,9 +400,9 @@ TurnAsyncSocket::createNewStunMessage(UInt16 stunclass, UInt16 method, bool addA
 }
 
 void
-TurnAsyncSocket::sendStunMessage(StunMessage* message, bool reTransmission)
+TurnAsyncSocket::sendStunMessage(StunMessage* message, bool reTransmission, unsigned int numRetransmits, unsigned int retrans_iterval_ms, const StunTuple* targetAddress)
 {
-#define REQUEST_BUFFER_SIZE 2048
+#define REQUEST_BUFFER_SIZE 4096
    boost::shared_ptr<DataBuffer> buffer = AsyncSocketBase::allocateBuffer(REQUEST_BUFFER_SIZE);
    unsigned int bufferSize;
    bufferSize = message->stunEncodeMessage((char*)buffer->data(), REQUEST_BUFFER_SIZE);
@@ -400,7 +413,7 @@ TurnAsyncSocket::sendStunMessage(StunMessage* message, bool reTransmission)
       // If message is a request, then start appropriate transaction and retranmission timers
       if(message->mClass == StunMessage::StunClassRequest)
       {
-         boost::shared_ptr<RequestEntry> requestEntry(new RequestEntry(mIOService, this, message));
+         boost::shared_ptr<RequestEntry> requestEntry(new RequestEntry(mIOService, this, message, numRetransmits, retrans_iterval_ms, targetAddress));
          mActiveRequestMap[message->mHeader.magicCookieAndTid] = requestEntry;
          requestEntry->startTimer();
       }
@@ -410,7 +423,14 @@ TurnAsyncSocket::sendStunMessage(StunMessage* message, bool reTransmission)
       }
    }
 
-   send(buffer);
+   if (targetAddress)
+   {
+      sendToUnframed(targetAddress->getAddress(), targetAddress->getPort(), buffer);
+   }
+   else
+   {
+      sendUnframed(buffer);
+   }
 }
 
 void 
@@ -422,7 +442,8 @@ TurnAsyncSocket::handleReceivedData(const asio::ip::address& address, unsigned s
       if((((*data)[0]) & 0xC0) == 0)
       {
          StunMessage* stunMsg = new StunMessage(mLocalBinding, 
-                                                StunTuple(mLocalBinding.getTransportType(), mAsyncSocketBase.getConnectedAddress(), mAsyncSocketBase.getConnectedPort()), 
+                                                //StunTuple(mLocalBinding.getTransportType(), mAsyncSocketBase.getConnectedAddress(), mAsyncSocketBase.getConnectedPort()), 
+                                                StunTuple(mLocalBinding.getTransportType(), address, port), 
                                                 &(*data)[0], data->size());
          if(stunMsg->isValid())
          {
@@ -494,11 +515,23 @@ TurnAsyncSocket::handleStunMessage(StunMessage& stunMessage)
    asio::error_code errorCode;
    if(stunMessage.isValid())
    {
+      if(stunMessage.mClass == StunMessage::StunClassSuccessResponse ||
+         stunMessage.mClass == StunMessage::StunClassErrorResponse)
+      {
       if(!stunMessage.checkMessageIntegrity(mHmacKey))
       {
          WarningLog(<< "TurnAsyncSocket::handleStunMessage: Stun message integrity is bad!");
          return asio::error_code(reTurn::BadMessageIntegrity, asio::error::misc_category);
       }
+      }
+      else
+      {
+         if(!stunMessage.checkMessageIntegrity(mLocalHmacKey))
+         {
+            WarningLog(<< "TurnAsyncSocket::handleStunMessage: Stun message integrity is bad!");
+            return asio::error_code(reTurn::BadMessageIntegrity, asio::error::misc_category);
+         }
+      }
 
       // Request is authenticated, process it
       switch(stunMessage.mClass)
@@ -589,57 +622,59 @@ TurnAsyncSocket::handleStunMessage(StunMessage& stunMessage)
             return asio::error_code(reTurn::StrayResponse, asio::error::misc_category);
          }
 
-         boost::shared_ptr<RequestEntry> requestEntry = it->second;
-         mActiveRequestMap.erase(it);
-         requestEntry->stopTimer();
-
-         // If a realm and nonce attributes are present and the response is a 401 or 438 (Nonce Expired), 
-         // then re-issue request with new auth attributes
-         if(stunMessage.mHasRealm &&
-            stunMessage.mHasNonce &&
-            stunMessage.mHasErrorCode && 
-            stunMessage.mErrorCode.errorClass == 4 &&
-            ((stunMessage.mErrorCode.number == 1 && mHmacKey.empty()) ||  // Note if 401 error then ensure we haven't already tried once - if we've tried then mHmacKey will be populated
-            stunMessage.mErrorCode.number == 38))
          {
-            mNonce = *stunMessage.mNonce;
-            mRealm = *stunMessage.mRealm;
-            stunMessage.calculateHmacKey(mHmacKey, mUsername, mRealm, mPassword);
+            boost::shared_ptr<RequestEntry> requestEntry = it->second;
+            mActiveRequestMap.erase(it);
+            requestEntry->stopTimer();
 
-            // Create a new transaction - by starting with old request
-            StunMessage* newRequest = requestEntry->mRequestMessage;
-            requestEntry->mRequestMessage = 0;  // clear out pointer in mActiveRequestMap so that it will not be deleted
+            // If a realm and nonce attributes are present and the response is a 401 or 438 (Nonce Expired), 
+            // then re-issue request with new auth attributes
+            if(stunMessage.mHasRealm &&
+               stunMessage.mHasNonce &&
+               stunMessage.mHasErrorCode && 
+               stunMessage.mErrorCode.errorClass == 4 &&
+               ((stunMessage.mErrorCode.number == 1 && mHmacKey.empty()) ||  // Note if 401 error then ensure we haven't already tried once - if we've tried then mHmacKey will be populated
+               stunMessage.mErrorCode.number == 38))
+            {
+               mNonce = *stunMessage.mNonce;
+               mRealm = *stunMessage.mRealm;
+               stunMessage.calculateHmacKey(mHmacKey, mUsername, mRealm, mPassword);
 
-            newRequest->createHeader(newRequest->mClass, newRequest->mMethod);  // updates TID
-            newRequest->mHasMessageIntegrity = true;
-            newRequest->setUsername(mUsername.c_str()); 
-            newRequest->mHmacKey = mHmacKey;
-            newRequest->setRealm(mRealm.c_str());
-            newRequest->setNonce(mNonce.c_str());
-            sendStunMessage(newRequest);
-            return errorCode;
-         }          
+               // Create a new transaction - by starting with old request
+               StunMessage* newRequest = requestEntry->mRequestMessage;
+               requestEntry->mRequestMessage = 0;  // clear out pointer in mActiveRequestMap so that it will not be deleted
 
-         switch (stunMessage.mMethod) 
-         {
-         case StunMessage::BindMethod:
-            errorCode = handleBindResponse(*requestEntry->mRequestMessage, stunMessage);
-            break;
-         case StunMessage::SharedSecretMethod:
-            errorCode = handleSharedSecretResponse(*requestEntry->mRequestMessage, stunMessage);
-            break;
-         case StunMessage::TurnAllocateMethod:
-            errorCode = handleAllocateResponse(*requestEntry->mRequestMessage, stunMessage);
-            break;
-         case StunMessage::TurnRefreshMethod:
-            errorCode = handleRefreshResponse(*requestEntry->mRequestMessage, stunMessage);
-            break;
-         case StunMessage::TurnChannelBindMethod:
-            errorCode = handleChannelBindResponse(*requestEntry->mRequestMessage, stunMessage);
-            break;
-         default:
-            // Unknown method - just ignore
-            break;
+               newRequest->createHeader(newRequest->mClass, newRequest->mMethod);  // updates TID
+               newRequest->mHasMessageIntegrity = true;
+               newRequest->setUsername(mUsername.c_str()); 
+               newRequest->mHmacKey = mHmacKey;
+               newRequest->setRealm(mRealm.c_str());
+               newRequest->setNonce(mNonce.c_str());
+               sendStunMessage(newRequest);
+               return errorCode;
+            }          
+
+            switch (stunMessage.mMethod) 
+            {
+            case StunMessage::BindMethod:
+               errorCode = handleBindResponse(*requestEntry->mRequestMessage, stunMessage);
+               break;
+            case StunMessage::SharedSecretMethod:
+               errorCode = handleSharedSecretResponse(*requestEntry->mRequestMessage, stunMessage);
+               break;
+            case StunMessage::TurnAllocateMethod:
+               errorCode = handleAllocateResponse(*requestEntry->mRequestMessage, stunMessage);
+               break;
+            case StunMessage::TurnRefreshMethod:
+               errorCode = handleRefreshResponse(*requestEntry->mRequestMessage, stunMessage);
+               break;
+            case StunMessage::TurnChannelBindMethod:
+               errorCode = handleChannelBindResponse(*requestEntry->mRequestMessage, stunMessage);
+               break;
+            default:
+               // Unknown method - just ignore
+               break;
+            }
          }
       }
       break;
@@ -700,13 +735,17 @@ TurnAsyncSocket::handleChannelBindResponse(StunMessage &request, StunMessage &re
       {
          // Remote Peer not found - discard
          WarningLog(<< "TurnAsyncSocket::handleChannelBindResponse: Received ChannelBindResponse for unknown channel (" << response.mTurnChannelNumber << ") - discarding");
-         return asio::error_code(reTurn::InvalidChannelNumberReceived, asio::error::misc_category);
+         asio::error_code ec(reTurn::InvalidChannelNumberReceived, asio::error::misc_category);
+         if(mTurnAsyncSocketHandler) mTurnAsyncSocketHandler->onChannelBindFailure(getSocketDescriptor(), ec);
+         return ec;
       }
 
       DebugLog(<< "TurnAsyncSocket::handleChannelBindResponse: Channel " << remotePeer->getChannel() << " is now bound to " << remotePeer->getPeerTuple());
       remotePeer->refresh();
       remotePeer->setChannelConfirmed();
       startChannelBindingTimer(remotePeer->getChannel());
+
+      if(mTurnAsyncSocketHandler) mTurnAsyncSocketHandler->onChannelBindSuccess(getSocketDescriptor(), remotePeer->getChannel());
    }
    else
    {
@@ -714,12 +753,16 @@ TurnAsyncSocket::handleChannelBindResponse(StunMessage &request, StunMessage &re
       if(response.mHasErrorCode)
       {
          ErrLog(<< "TurnAsyncSocket::handleChannelBindResponse: Received ChannelBindResponse error: " << response.mErrorCode.errorClass * 100 + response.mErrorCode.number);
-         return asio::error_code(response.mErrorCode.errorClass * 100 + response.mErrorCode.number, asio::error::misc_category);
+         asio::error_code ec(response.mErrorCode.errorClass * 100 + response.mErrorCode.number, asio::error::misc_category);
+         if(mTurnAsyncSocketHandler) mTurnAsyncSocketHandler->onChannelBindFailure(getSocketDescriptor(), ec);
+         return ec;
       }
       else
       {
          ErrLog(<< "TurnAsyncSocket::handleChannelBindResponse: Received ChannelBindResponse error but no error code attribute found.");
-         return asio::error_code(MissingAttributes, asio::error::misc_category);
+         asio::error_code ec(MissingAttributes, asio::error::misc_category);
+         if(mTurnAsyncSocketHandler) mTurnAsyncSocketHandler->onChannelBindFailure(getSocketDescriptor(), ec);
+         return ec;
       }
    }
    return asio::error_code();
@@ -777,8 +820,23 @@ TurnAsyncSocket::handleBindRequest(StunMessage& stunMessage)
    // Add Software Attribute
    response->setSoftware(SOFTWARE_STRING);
 
+   // If the request contained MESSAGE-INTEGRITY, then the response needs to as well
+   if (stunMessage.mHasMessageIntegrity)
+   {
+      response->mHasMessageIntegrity = true;
+      response->mHmacKey = mLocalHmacKey;
+   }
+
+   if (stunMessage.mHasIceControlled || stunMessage.mHasIceControlling || stunMessage.mHasIcePriority)
+   {
+      response->mHasFingerprint = true;
+   }
+
    // send bindResponse to local client
-   sendStunMessage(response);
+   DebugLog(<< "Sending response to BIND to " << stunMessage.mRemoteTuple);
+   sendStunMessage(response, false, UDP_MAX_RETRANSMITS, DEFAULT_RETRANS_INTERVAL_MS, &(stunMessage.mRemoteTuple));
+   
+   if(mTurnAsyncSocketHandler) mTurnAsyncSocketHandler->onIncomingBindRequestProcessed(getSocketDescriptor(), stunMessage.mRemoteTuple);
 
    return asio::error_code();
 }
@@ -800,21 +858,21 @@ TurnAsyncSocket::handleBindResponse(StunMessage &request, StunMessage &response)
       }
       else
       {
-         if(mTurnAsyncSocketHandler) mTurnAsyncSocketHandler->onBindFailure(getSocketDescriptor(), asio::error_code(MissingAttributes, asio::error::misc_category));
+         if(mTurnAsyncSocketHandler) mTurnAsyncSocketHandler->onBindFailure(getSocketDescriptor(), asio::error_code(MissingAttributes, asio::error::misc_category), response.mRemoteTuple);
          return asio::error_code(MissingAttributes, asio::error::misc_category);
       }
-      if(mTurnAsyncSocketHandler) mTurnAsyncSocketHandler->onBindSuccess(getSocketDescriptor(), reflexiveTuple);
+      if(mTurnAsyncSocketHandler) mTurnAsyncSocketHandler->onBindSuccess(getSocketDescriptor(), reflexiveTuple, response.mRemoteTuple);
    }
    else
    {
       // Check if success or not
       if(response.mHasErrorCode)
       {
-         if(mTurnAsyncSocketHandler) mTurnAsyncSocketHandler->onBindFailure(getSocketDescriptor(), asio::error_code(response.mErrorCode.errorClass * 100 + response.mErrorCode.number, asio::error::misc_category));
+         if(mTurnAsyncSocketHandler) mTurnAsyncSocketHandler->onBindFailure(getSocketDescriptor(), asio::error_code(response.mErrorCode.errorClass * 100 + response.mErrorCode.number, asio::error::misc_category), response.mRemoteTuple);
       }
       else
       {
-         if(mTurnAsyncSocketHandler) mTurnAsyncSocketHandler->onBindFailure(getSocketDescriptor(), asio::error_code(MissingAttributes, asio::error::misc_category));
+         if(mTurnAsyncSocketHandler) mTurnAsyncSocketHandler->onBindFailure(getSocketDescriptor(), asio::error_code(MissingAttributes, asio::error::misc_category), response.mRemoteTuple);
          return asio::error_code(MissingAttributes, asio::error::misc_category);
       }
    }
@@ -952,46 +1010,63 @@ void
 TurnAsyncSocket::send(const char* buffer, unsigned int size)
 {
    boost::shared_ptr<DataBuffer> data(new DataBuffer(buffer, size));
-   mGuards.push(mAsyncSocketBase.shared_from_this());
-   mIOService.post(boost::bind(&TurnAsyncSocket::doSend, this, data));
-}
-
-void
-TurnAsyncSocket::doSend(boost::shared_ptr<DataBuffer>& data)
-{
-   GuardReleaser guardReleaser(mGuards);
-
-   // Allow raw data to be sent if there is no allocation
-   if(!mHaveAllocation)
-   {
-      send(data);
-      return;
-   }
-
-   return sendTo(*mActiveDestination, data);
+   sendFramed(data);
 }
 
 void 
 TurnAsyncSocket::sendTo(const asio::ip::address& address, unsigned short port, const char* buffer, unsigned int size)
 {
    boost::shared_ptr<DataBuffer> data(new DataBuffer(buffer, size));
-   mGuards.push(mAsyncSocketBase.shared_from_this());
-   mIOService.post(boost::bind(&TurnAsyncSocket::doSendTo, this, address, port, data));
+   sendToFramed(address, port, data);
 }
 
 void 
-TurnAsyncSocket::doSendTo(const asio::ip::address& address, unsigned short port, boost::shared_ptr<DataBuffer>& data)
+TurnAsyncSocket::sendFramed(boost::shared_ptr<DataBuffer>& data)
 {
-   GuardReleaser guardReleaser(mGuards);
+   mIOService.dispatch(weak_bind<AsyncSocketBase, void()>( mAsyncSocketBase.shared_from_this(), boost::bind(&TurnAsyncSocket::doSendFramed, this, data)));
+}
 
-   // Allow raw data to be sent if there is no allocation
-   if(!mHaveAllocation)
+void 
+TurnAsyncSocket::sendToFramed(const asio::ip::address& address, unsigned short port, boost::shared_ptr<DataBuffer>& data)
+{
+   mIOService.dispatch(weak_bind<AsyncSocketBase, void()>( mAsyncSocketBase.shared_from_this(), boost::bind(&TurnAsyncSocket::doSendToFramed, this, address, port, data)));
+}
+
+void 
+TurnAsyncSocket::sendUnframed(boost::shared_ptr<DataBuffer>& data)
+{
+   StunTuple destination(mLocalBinding.getTransportType(), mAsyncSocketBase.getConnectedAddress(), mAsyncSocketBase.getConnectedPort());
+   mAsyncSocketBase.send(destination, data);
+}
+
+void 
+TurnAsyncSocket::sendToUnframed(const asio::ip::address& address, unsigned short port, boost::shared_ptr<DataBuffer>& data)
+{
+   StunTuple destination(mLocalBinding.getTransportType(), address, port);
+   mAsyncSocketBase.send(destination, data);
+}
+
+void 
+TurnAsyncSocket::doSendFramed(boost::shared_ptr<DataBuffer>& data)
+{
+   if(mActiveDestination)
    {
-      StunTuple destination(mLocalBinding.getTransportType(), address, port);
-      mAsyncSocketBase.send(destination, data);
-      return;
+      sendToRemotePeer(*mActiveDestination, data);
    }
+   else if(mAsyncSocketBase.isConnected())
+   {
+      DebugLog(<<"Sending to connected peer");
+      sendToUnframed(mAsyncSocketBase.getConnectedAddress(), mAsyncSocketBase.getConnectedPort(), data);
+   }
+   else
+   {
+      DebugLog(<<"no allocation, can't send!");
+   }
+}
 
+void 
+TurnAsyncSocket::doSendToFramed(const asio::ip::address& address, unsigned short port, boost::shared_ptr<DataBuffer>& data)
+{
    // Setup Remote Peer 
    StunTuple remoteTuple(mRelayTransportType, address, port);
    RemotePeer* remotePeer = mChannelManager.findRemotePeerByPeerAddress(remoteTuple);
@@ -1002,16 +1077,16 @@ TurnAsyncSocket::doSendTo(const asio::ip::address& address, unsigned short port,
       assert(remotePeer);
       doChannelBinding(*remotePeer);
    }
-   return sendTo(*remotePeer, data);
+   return sendToRemotePeer(*remotePeer, data);
 }
 
 void
-TurnAsyncSocket::sendTo(RemotePeer& remotePeer, boost::shared_ptr<DataBuffer>& data)
+TurnAsyncSocket::sendToRemotePeer(RemotePeer& remotePeer, boost::shared_ptr<DataBuffer>& data)
 {
    if(remotePeer.isChannelConfirmed())
    {
       // send framed data to active destination
-      send(remotePeer.getChannel(), data);
+      sendOverChannel(remotePeer.getChannel(), data);
       //InfoLog( << "TurnAsyncSocket::sendTo: using channel " << remotePeer.getChannel() << " to send " << data->size() << " bytes.");
    }
    else
@@ -1034,21 +1109,18 @@ TurnAsyncSocket::sendTo(RemotePeer& remotePeer, boost::shared_ptr<DataBuffer>& d
 void
 TurnAsyncSocket::connect(const std::string& address, unsigned short port)
 {
-   mAsyncSocketBase.connect(address,port);
+   mAsyncSocketBase.connect(address, port);
 }
 
 void
 TurnAsyncSocket::close()
 {
-   mGuards.push(mAsyncSocketBase.shared_from_this());
-   mIOService.post(boost::bind(&TurnAsyncSocket::doClose, this));
+   mIOService.post(weak_bind<AsyncSocketBase, void()>(mAsyncSocketBase.shared_from_this(), boost::bind(&TurnAsyncSocket::doClose, this)));
 }
 
 void
 TurnAsyncSocket::doClose()
 {
-   GuardReleaser guardReleaser(mGuards);
-
    // If we have an allocation over UDP then we should send a refresh with lifetime 0 to destroy the allocation
    // Note:  For TCP and TLS, the socket disconnection will destroy the allocation automatically
    if(mHaveAllocation && mLocalBinding.getTransportType() == StunTuple::UDP)
@@ -1087,14 +1159,7 @@ TurnAsyncSocket::turnReceive()
 }
 
 void 
-TurnAsyncSocket::send(boost::shared_ptr<DataBuffer>& data)
-{
-   StunTuple destination(mLocalBinding.getTransportType(), mAsyncSocketBase.getConnectedAddress(), mAsyncSocketBase.getConnectedPort());
-   mAsyncSocketBase.send(destination, data);
-}
-
-void 
-TurnAsyncSocket::send(unsigned short channel, boost::shared_ptr<DataBuffer>& data)
+TurnAsyncSocket::sendOverChannel(unsigned short channel, boost::shared_ptr<DataBuffer>& data)
 {
    StunTuple destination(mLocalBinding.getTransportType(), mAsyncSocketBase.getConnectedAddress(), mAsyncSocketBase.getConnectedPort());
    mAsyncSocketBase.send(destination, channel, data);
@@ -1102,33 +1167,45 @@ TurnAsyncSocket::send(unsigned short channel, boost::shared_ptr<DataBuffer>& dat
 
 TurnAsyncSocket::RequestEntry::RequestEntry(asio::io_service& ioService, 
                                             TurnAsyncSocket* turnAsyncSocket, 
-                                            StunMessage* requestMessage) : 
+                                            StunMessage* requestMessage,
+                                            unsigned int rc,
+                                            unsigned int retrans_interval_ms,
+                                            const StunTuple* dest)
+                                             : 
    mIOService(ioService), 
    mTurnAsyncSocket(turnAsyncSocket), 
    mRequestMessage(requestMessage), 
    mRequestTimer(ioService),
-   mRequestsSent(1)
+   mRequestsSent(1),
+   mDest(dest ? new StunTuple(dest->getTransportType(), dest->getAddress(), dest->getPort()) : 0),
+   mRc(rc),
+   mRetransIntervalMs(retrans_interval_ms)
 {
+   //std::cout << "RequestEntry::RequestEntry() " << mRequestMessage->mHeader.magicCookieAndTid << " dest: " << (mDest ? dest->getPort() : 0) << std::endl;
+
    mTimeout = mTurnAsyncSocket->mLocalBinding.getTransportType() == StunTuple::UDP ? UDP_RT0 : TCP_RESPONSE_TIME;
 }
 
 void
 TurnAsyncSocket::RequestEntry::startTimer()
 {
+   //std::cout << "RequestEntry::startTimer() " << mTimeout << " " << mRequestMessage->mHeader.magicCookieAndTid << std::endl;
    // start the request timer
    mRequestTimer.expires_from_now(boost::posix_time::milliseconds(mTimeout));  
-   mRequestTimer.async_wait(boost::bind(&TurnAsyncSocket::RequestEntry::requestTimerExpired, shared_from_this(), asio::placeholders::error));
+   mRequestTimer.async_wait(weak_bind<RequestEntry, void(const asio::error_code&)>(shared_from_this(), boost::bind(&TurnAsyncSocket::RequestEntry::requestTimerExpired, this, asio::placeholders::error)));
 }
 
 void
 TurnAsyncSocket::RequestEntry::stopTimer()
 {
+   //std::cout << "RequestEntry::stopTimer() " << mRequestMessage->mHeader.magicCookieAndTid << std::endl;
    // stop the request timer
    mRequestTimer.cancel();
 }
 
 TurnAsyncSocket::RequestEntry::~RequestEntry() 
 { 
+   //std::cout << "RequestEntry::~RequestEntry() " << mRequestMessage->mHeader.magicCookieAndTid << std::endl;
    delete mRequestMessage; 
    stopTimer();
 }
@@ -1138,13 +1215,17 @@ TurnAsyncSocket::RequestEntry::requestTimerExpired(const asio::error_code& e)
 {
    if(!e && mRequestMessage)  // Note:  There is a race condition with clearing out of mRequestMessage when 401 is received - check that mRequestMessage is not 0 avoids any resulting badness
    {
-      if(mTurnAsyncSocket->mLocalBinding.getTransportType() != StunTuple::UDP || mRequestsSent == UDP_MAX_RETRANSMITS)
+      //std::cout << "RequestEntry::requestTimerExpired() " << mRequestMessage->mHeader.magicCookieAndTid << std::endl;
+
+      if(mTurnAsyncSocket->mLocalBinding.getTransportType() != StunTuple::UDP || mRequestsSent == mRc)
       {
          mTurnAsyncSocket->requestTimeout(mRequestMessage->mHeader.magicCookieAndTid);
          return;
       }
       // timed out and should retransmit - calculate next timeout
-      if(mRequestsSent == UDP_MAX_RETRANSMITS - 1)
+      if(mRetransIntervalMs == DEFAULT_RETRANS_INTERVAL_MS)
+      {
+         if(mRequestsSent == mRc - 1)
       {
           mTimeout = UDP_FINAL_REQUEST_TIME;
       } 
@@ -1152,10 +1233,16 @@ TurnAsyncSocket::RequestEntry::requestTimerExpired(const asio::error_code& e)
       {
           mTimeout = (mTimeout*2);
       }
+      }
+      else
+      {
+         mTimeout = mRetransIntervalMs;
+      }
+
       // retransmit
       DebugLog(<< "RequestEntry::requestTimerExpired: retransmitting...");
       mRequestsSent++;
-      mTurnAsyncSocket->sendStunMessage(mRequestMessage, true);
+      mTurnAsyncSocket->sendStunMessage(mRequestMessage, true, UDP_MAX_RETRANSMITS, DEFAULT_RETRANS_INTERVAL_MS, mDest);
 
       startTimer();
    }
@@ -1168,12 +1255,12 @@ TurnAsyncSocket::requestTimeout(UInt128 tid)
    if(it != mActiveRequestMap.end())
    {
       boost::shared_ptr<RequestEntry> requestEntry = it->second;
-      mActiveRequestMap.erase(it);
+      mActiveRequestMap.erase(tid);
 
       switch(requestEntry->mRequestMessage->mMethod)
       {
       case StunMessage::BindMethod:
-         if(mTurnAsyncSocketHandler) mTurnAsyncSocketHandler->onBindFailure(getSocketDescriptor(), asio::error_code(reTurn::ResponseTimeout, asio::error::misc_category));
+         if(mTurnAsyncSocketHandler) mTurnAsyncSocketHandler->onBindFailure(getSocketDescriptor(), asio::error_code(reTurn::ResponseTimeout, asio::error::misc_category), (requestEntry->mDest ? *(requestEntry->mDest) : StunTuple()));
          break;
       case StunMessage::SharedSecretMethod:
          if(mTurnAsyncSocketHandler) mTurnAsyncSocketHandler->onSharedSecretFailure(getSocketDescriptor(), asio::error_code(reTurn::ResponseTimeout, asio::error::misc_category));
@@ -1189,6 +1276,10 @@ TurnAsyncSocket::requestTimeout(UInt128 tid)
             actualClose();
          }
          break;
+      case StunMessage::TurnChannelBindMethod:  
+         // Note:  ChannelBind can happen after SetActiveDestination, after a sendTo, or during a channel bind refresh
+         if(mTurnAsyncSocketHandler) mTurnAsyncSocketHandler->onChannelBindFailure(getSocketDescriptor(), asio::error_code(reTurn::ResponseTimeout, asio::error::misc_category));
+         break;
       default:
          assert(false);
       }
@@ -1211,8 +1302,7 @@ void
 TurnAsyncSocket::startAllocationTimer()
 {
    mAllocationTimer.expires_from_now(boost::posix_time::seconds((mLifetime*5)/8));  // Allocation refresh should sent before 3/4 lifetime - use 5/8 lifetime 
-   mGuards.push(mAsyncSocketBase.shared_from_this());
-   mAllocationTimer.async_wait(boost::bind(&TurnAsyncSocket::allocationTimerExpired, this, asio::placeholders::error));
+   mAllocationTimer.async_wait(weak_bind<AsyncSocketBase, void(const asio::error_code&)>(mAsyncSocketBase.shared_from_this(), boost::bind(&TurnAsyncSocket::allocationTimerExpired, this, asio::placeholders::error)));
 }
 
 void
@@ -1225,16 +1315,8 @@ void
 TurnAsyncSocket::allocationTimerExpired(const asio::error_code& e)
 {
    if(!e)
-   {
       doRefreshAllocation(mLifetime);
    }
-   else
-   {
-      // Note:  only release guard if not calling doRefreshAllocation - since
-      // doRefreshAllocation will release the guard
-      GuardReleaser guardReleaser(mGuards);
-   }
-}
 
 void
 TurnAsyncSocket::startChannelBindingTimer(unsigned short channel)
@@ -1248,8 +1330,7 @@ TurnAsyncSocket::startChannelBindingTimer(unsigned short channel)
       it = ret.first;
    }
    it->second->expires_from_now(boost::posix_time::seconds(TURN_CHANNEL_BINDING_REFRESH_SECONDS));  
-   mGuards.push(mAsyncSocketBase.shared_from_this());
-   it->second->async_wait(boost::bind(&TurnAsyncSocket::channelBindingTimerExpired, this, asio::placeholders::error, channel));
+   it->second->async_wait(weak_bind<AsyncSocketBase, void(const asio::error_code&)>( mAsyncSocketBase.shared_from_this(), boost::bind(&TurnAsyncSocket::channelBindingTimerExpired, this, asio::placeholders::error, channel)));
 }
 
 void
@@ -1268,7 +1349,6 @@ TurnAsyncSocket::cancelChannelBindingTimers()
 void 
 TurnAsyncSocket::channelBindingTimerExpired(const asio::error_code& e, unsigned short channel)
 {
-   GuardReleaser guardReleaser(mGuards);
    if(!e)
    {
       RemotePeer* remotePeer = mChannelManager.findRemotePeerByChannel(channel);
@@ -1279,6 +1359,12 @@ TurnAsyncSocket::channelBindingTimerExpired(const asio::error_code& e, unsigned
    }
 }
 
+void 
+TurnAsyncSocket::setOnBeforeSocketClosedFp(boost::function<void(unsigned int)> fp)
+{
+   mAsyncSocketBase.setOnBeforeSocketClosedFp(fp);
+}
+
 } // namespace
 
 
