@@ -29,7 +29,7 @@ using namespace std;
 
 #define RESIPROCATE_SUBSYSTEM FlowManagerSubsystem::FLOWMANAGER
 
-char* srtp_error_string(err_status_t error)
+const char* srtp_error_string(err_status_t error)
 {
    switch(error)
    {
@@ -346,7 +346,7 @@ Flow::receiveFrom(const asio::ip::address& address, unsigned short port, char* b
          return asio::error_code(flowmanager::ReceiveTimeout, asio::error::misc_category);
       }
 
-      recvTimeout = timeout ? timeout - (Timer::getTimeMs() - startTime) : 0;
+      recvTimeout = timeout ? (unsigned int)(timeout - (Timer::getTimeMs() - startTime)) : 0;
       if(timeout != 0 && recvTimeout <= 0)
       {
          // timeout
@@ -638,7 +638,7 @@ Flow::onSharedSecretFailure(unsigned int socketDesc, const asio::error_code& e)
 }
 
 void 
-Flow::onBindSuccess(unsigned int socketDesc, const StunTuple& reflexiveTuple)
+Flow::onBindSuccess(unsigned int socketDesc, const StunTuple& reflexiveTuple, const StunTuple& stunServerTuple)
 {
    InfoLog(<< "Flow::onBindingSuccess: socketDesc=" << socketDesc << ", reflexive=" << reflexiveTuple << ", componentId=" << mComponentId);
    {
@@ -649,7 +649,7 @@ Flow::onBindSuccess(unsigned int socketDesc, const StunTuple& reflexiveTuple)
    mMediaStream.onFlowReady(mComponentId);
 }
 void 
-Flow::onBindFailure(unsigned int socketDesc, const asio::error_code& e)
+Flow::onBindFailure(unsigned int socketDesc, const asio::error_code& e, const StunTuple& stunServerTuple)
 {
    WarningLog(<< "Flow::onBindingFailure: socketDesc=" << socketDesc << " error=" << e.value() << "(" << e.message() << "), componentId=" << mComponentId );
    changeFlowState(Connected);
@@ -724,6 +724,24 @@ Flow::onClearActiveDestinationFailure(unsigned int socketDesc, const asio::error
    WarningLog(<< "Flow::onClearActiveDestinationFailure: socketDesc=" << socketDesc << " error=" << e.value() << "(" << e.message() << "), componentId=" << mComponentId );
 }
 
+void 
+Flow::onChannelBindRequestSent(unsigned int socketDesc, unsigned short channelNumber)
+{
+   InfoLog(<< "Flow::onChannelBindRequestSent: socketDesc=" << socketDesc << ", channelNumber=" << channelNumber << ", componentId=" << mComponentId);
+}
+
+void 
+Flow::onChannelBindSuccess(unsigned int socketDesc, unsigned short channelNumber)
+{
+   InfoLog(<< "Flow::onChannelBindSuccess: socketDesc=" << socketDesc << ", channelNumber=" << channelNumber << ", componentId=" << mComponentId);
+}
+
+void 
+Flow::onChannelBindFailure(unsigned int socketDesc, const asio::error_code& e)
+{
+   WarningLog(<< "Flow::onChannelBindFailure: socketDesc=" << socketDesc << " error=" << e.value() << "(" << e.message() << "), componentId=" << mComponentId );
+}
+
 void 
 Flow::onSendSuccess(unsigned int socketDesc)
 {
@@ -797,6 +815,13 @@ Flow::onReceiveFailure(unsigned int socketDesc, const asio::error_code& e)
    }
 }
 
+void 
+Flow::onIncomingBindRequestProcessed(unsigned int socketDesc, const StunTuple& sourceTuple)
+{
+   InfoLog(<< "Flow::onIncomingBindRequestProcessed: socketDesc=" << socketDesc << ", sourceTuple=" << sourceTuple );
+   // TODO - handle
+}
+
 void 
 Flow::changeFlowState(FlowState newState)
 {
@@ -804,7 +829,7 @@ Flow::changeFlowState(FlowState newState)
    mFlowState = newState;
 }
 
-char*
+const char*
 Flow::flowStateToString(FlowState state)
 {
    switch(state)
