@@ -1,6 +1,3 @@
-// testXmlRpc.cpp : Defines the entry point for the console application.
-//
-
 #include <rutil/DnsUtil.hxx>
 #include <rutil/BaseException.hxx>
 #include <rutil/XMLCursor.hxx>
@@ -29,35 +26,42 @@ main (int argc, char** argv)
    struct sockaddr_in localAddr, servAddr;
    struct hostent *h;
 
-   if(argc != 2 && argc != 4) 
+   const char* host;
+   short port;
+   int cmdIndex;
+
+   // Look for forward slash in args 1 or 3
+   if(argc >= 2 && argv[1][0] == '/')
    {
-      cerr << "usage: " << argv[0] <<" [<server> <port>] <command> [<parm>=<value>]" << endl;
-      cerr << "  Valid Commands are:" << endl;
-      cerr << "  GetStackInfo - retrieves low level information about the stack state" << endl;
-      cerr << "  GetStackStats - retrieves a dump of the stack statistics" << endl;
-      cerr << "  ResetStackStats - resets all cumulative stack statistics to zero" << endl;
-      cerr << "  LogDnsCache - causes the DNS cache contents to be written to the resip logs" << endl;
-      cerr << "  ClearDnsCache - empties the stacks DNS cache" << endl;
-      cerr << "  GetDnsCache - retrieves the DNS cache contents" << endl;
-      cerr << "  GetCongestionStats - retrieves the stacks congestion manager stats and state" << endl;
-      cerr << "  SetCongestionTolerance metric=<SIZE|WAIT_TIME|TIME_DEPTH> maxTolerance=<value>" << endl;
-      cerr << "                         [fifoDescription=<desc>] - sets congestion tolerances" << endl;
-      cerr << "  Shutdown - signal the proxy to shut down." << endl;
-      cerr << "  Restart - signal the proxy to restart - leaving active registrations in place." << endl;
-      cerr << "  GetProxyConfig - retrieves the all of configuration being used by the proxy" << endl;
-      exit(1);
+       host = "127.0.0.1";
+       port = 5081;
+       cmdIndex=1;
    }
-
-   const char* host = "127.0.0.1";
-   short port = 5081;
-   int cmdIndex=1;
-
-   if(argc == 4)
+   else if(argc >= 4 && argv[3][0] == '/')
    {
       host = argv[1];
       port = (short)atoi(argv[2]);
       cmdIndex = 3;
    }
+   else
+   {
+      cerr << "usage: " << argv[0] <<" [<server> <port>] /<command> [<parm>=<value>]" << endl;
+      cerr << "  Valid Commands are:" << endl;
+      cerr << "  /GetStackInfo - retrieves low level information about the stack state" << endl;
+      cerr << "  /GetStackStats - retrieves a dump of the stack statistics" << endl;
+      cerr << "  /ResetStackStats - resets all cumulative stack statistics to zero" << endl;
+      cerr << "  /LogDnsCache - causes the DNS cache contents to be written to the resip logs" << endl;
+      cerr << "  /ClearDnsCache - empties the stacks DNS cache" << endl;
+      cerr << "  /GetDnsCache - retrieves the DNS cache contents" << endl;
+      cerr << "  /GetCongestionStats - retrieves the stacks congestion manager stats and state" << endl;
+      cerr << "  /SetCongestionTolerance metric=<SIZE|WAIT_TIME|TIME_DEPTH> maxTolerance=<value>" << endl;
+      cerr << "                          [fifoDescription=<desc>] - sets congestion tolerances" << endl;
+      cerr << "  /Shutdown - signal the proxy to shut down." << endl;
+      cerr << "  /Restart - signal the proxy to restart - leaving active registrations in place." << endl;
+      cerr << "  /GetProxyConfig - retrieves the all of configuration file settings currently" << endl;
+      cerr << "                    being used by the proxy" << endl;
+      exit(1);
+   }
 
    h = gethostbyname(host);
    if(h==0) 
@@ -103,9 +107,10 @@ main (int argc, char** argv)
       exit(1);
    }
 
+   Data command(&argv[cmdIndex][1]); // [1] skips leading slash
    Data request(1024, Data::Preallocate);
    request += "<";
-   request += argv[cmdIndex];
+   request += command;
    request += ">\r\n  <Request>\r\n";
    for(int i = cmdIndex+1; i < argc; i++)
    {
@@ -133,7 +138,7 @@ main (int argc, char** argv)
    }
    request += "  </Request>\r\n";
    request += "</";
-   request += argv[cmdIndex];
+   request += command;
    request += ">\r\n";
 
    //cout << "Sending:\r\n" << request << endl;
@@ -180,7 +185,7 @@ main (int argc, char** argv)
                   }
                   if(xml.firstChild())
                   {
-                     text = xml.getValue();
+                     text = xml.getValue().xmlCharDataDecode();
                      xml.parent();
                   }
                   if(code >= 200 && code < 300)
@@ -219,3 +224,58 @@ main (int argc, char** argv)
       }
    }
 }
+
+
+/* ====================================================================
+ * The Vovida Software License, Version 1.0 
+ * 
+ * Copyright (c) 2000-2012
+ * 
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ * 
+ * 3. The names "VOCAL", "Vovida Open Communication Application Library",
+ *    and "Vovida Open Communication Application Library (VOCAL)" must
+ *    not be used to endorse or promote products derived from this
+ *    software without prior written permission. For written
+ *    permission, please contact vocal@vovida.org.
+ *
+ * 4. Products derived from this software may not be called "VOCAL", nor
+ *    may "VOCAL" appear in their name, without prior written
+ *    permission of Vovida Networks, Inc.
+ * 
+ * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
+ * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
+ * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
+ * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
+ * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
+ * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
+ * DAMAGE.
+ * 
+ * ====================================================================
+ * 
+ * This software consists of voluntary contributions made by Vovida
+ * Networks, Inc. and many individuals on behalf of Vovida Networks,
+ * Inc.  For more information on Vovida Networks, Inc., please see
+ * <http://www.vovida.org/>.
+ *
+ */
+/*
+ * vi: set shiftwidth=3 expandtab:
+ */
+
