@@ -51,6 +51,25 @@ ConversationManager::ConversationManager(bool localAudioEnabled, MediaInterfaceM
   mMediaFactory(0),
   mBridgeMixer(0),
   mSipXTOSValue(0)
+{
+   init();
+}
+
+ConversationManager::ConversationManager(bool localAudioEnabled, MediaInterfaceMode mediaInterfaceMode, int defaultSampleRate, int maxSampleRate)
+: mUserAgent(0),
+  mCurrentConversationHandle(1),
+  mCurrentParticipantHandle(1),
+  mLocalAudioEnabled(localAudioEnabled),
+  mMediaInterfaceMode(mediaInterfaceMode),
+  mMediaFactory(0),
+  mBridgeMixer(0),
+  mSipXTOSValue(0)
+{
+   init(defaultSampleRate, maxSampleRate);
+}
+
+void
+ConversationManager::init(int defaultSampleRate, int maxSampleRate)
 {
 #ifdef _DEBUG
    UtlString codecPaths[] = {".", "../../../../sipXtapi/sipXmediaLib/bin"};
@@ -65,11 +84,11 @@ ConversationManager::ConversationManager(bool localAudioEnabled, MediaInterfaceM
    {
       OsConfigDb sipXconfig;
       sipXconfig.set("PHONESET_MAX_ACTIVE_CALLS_ALLOWED",300);  // This controls the maximum number of flowgraphs allowed - default is 16
-      mMediaFactory = sipXmediaFactoryFactory(&sipXconfig, 0, 0, 0, mLocalAudioEnabled);
+      mMediaFactory = sipXmediaFactoryFactory(&sipXconfig, 0, defaultSampleRate, maxSampleRate, mLocalAudioEnabled);
    }
    else
    {
-      mMediaFactory = sipXmediaFactoryFactory(NULL, 0, 0, 0, mLocalAudioEnabled);
+      mMediaFactory = sipXmediaFactoryFactory(NULL, 0, defaultSampleRate, maxSampleRate, mLocalAudioEnabled);
    }
 
    // Create MediaInterface
@@ -185,10 +204,17 @@ ConversationManager::joinConversation(ConversationHandle sourceConvHandle, Conve
 
 ParticipantHandle 
 ConversationManager::createRemoteParticipant(ConversationHandle convHandle, const NameAddr& destination, ParticipantForkSelectMode forkSelectMode)
+{
+   SharedPtr<UserProfile> profile;
+   return createRemoteParticipant(convHandle, destination, forkSelectMode, profile, std::multimap<resip::Data,resip::Data>());
+}
+
+ParticipantHandle
+ConversationManager::createRemoteParticipant(ConversationHandle convHandle, const resip::NameAddr& destination, ParticipantForkSelectMode forkSelectMode, SharedPtr<UserProfile>& callerProfile, const std::multimap<resip::Data,resip::Data>& extraHeaders)
 {
    ParticipantHandle partHandle = getNewParticipantHandle();
 
-   CreateRemoteParticipantCmd* cmd = new CreateRemoteParticipantCmd(this, partHandle, convHandle, destination, forkSelectMode);
+   CreateRemoteParticipantCmd* cmd = new CreateRemoteParticipantCmd(this, partHandle, convHandle, destination, forkSelectMode, callerProfile, extraHeaders);
    post(cmd);
 
    return partHandle;
@@ -570,8 +596,11 @@ ConversationManager::buildSessionCapabilities(const resip::Data& ipaddress, unsi
             sdpcodec->getEncodingName(mimeSubType);
             //mimeSubType.toUpper();
             
-            SdpContents::Session::Codec codec(mimeSubType.data(), sdpcodec->getSampleRate());
-            codec.payloadType() = sdpcodec->getCodecPayloadFormat();
+            SdpContents::Session::Codec codec(mimeSubType.data(), sdpcodec->getCodecPayloadFormat(), sdpcodec->getSampleRate());
+            if(sdpcodec->getNumChannels() > 1)
+            {
+               codec.encodingParameters() = Data(sdpcodec->getNumChannels());
+            }
 
             // Check for telephone-event and add fmtp manually
             if(mimeSubType.compareTo("telephone-event", UtlString::ignoreCase) == 0)
@@ -999,8 +1028,15 @@ ConversationManager::onNewSubscriptionFromRefer(ServerSubscriptionHandle ss, con
 
          // Notify application
          ConversationProfile* profile = dynamic_cast<ConversationProfile*>(ss->getUserProfile().get());
-         assert(profile);
-         onRequestOutgoingParticipant(participant->getParticipantHandle(), msg, *profile);
+         if(profile)
+         {
+            onRequestOutgoingParticipant(participant->getParticipantHandle(), msg, *profile);
+         }
+         else
+         {
+            // FIXME - could we do something else here?
+            WarningLog(<<"not an instance of ConversationProfile, not calling onRequestOutgoingParticipant");
+         }
       }
       else
       {
@@ -1138,7 +1174,15 @@ ConversationManager::onReceivedRequest(ServerOutOfDialogReqHandle ood, const Sip
             // Notify application
             ConversationProfile* profile = dynamic_cast<ConversationProfile*>(ood->getUserProfile().get());
             assert(profile);
-            onRequestOutgoingParticipant(participant->getParticipantHandle(), msg, *profile);
+            if(profile)
+            {
+               onRequestOutgoingParticipant(participant->getParticipantHandle(), msg, *profile);
+            }
+            else
+            {
+               // FIXME - could we do something else here?
+               WarningLog(<<"not an instance of ConversationProfile, not calling onRequestOutgoingParticipant");
+            }
          }
          else
          {
