@@ -19,6 +19,7 @@
 #include <rutil/Logger.hxx>
 #include <rutil/DnsUtil.hxx>
 #include <rutil/Random.hxx>
+#include <resip/stack/DtmfPayloadContents.hxx>
 #include <resip/stack/SipFrag.hxx>
 #include <resip/stack/ExtensionHeader.hxx>
 #include <resip/dum/DialogUsageManager.hxx>
@@ -37,6 +38,20 @@ using namespace std;
 
 #define RESIPROCATE_SUBSYSTEM ReconSubsystem::RECON
 
+/* Technically, there are a range of features that need to be implemented
+   to be fully (S)AVPF compliant.
+   However, it is speculated that (S)AVPF peers will communicate with legacy
+   systems that just fudge the RTP/SAVPF protocol in their SDP.  Enabling
+   this define allows such behavior to be tested. 
+
+   http://www.ietf.org/mail-archive/web/rtcweb/current/msg01145.html
+   "1) RTCWEB end-point will always signal AVPF or SAVPF. I signalling
+   gateway to legacy will change that by removing the F to AVP or SAVP."
+
+   http://www.ietf.org/mail-archive/web/rtcweb/current/msg04380.html
+*/
+//#define RTP_SAVPF_FUDGE
+
 // UAC
 RemoteParticipant::RemoteParticipant(ParticipantHandle partHandle,
                                      ConversationManager& conversationManager, 
@@ -107,9 +122,20 @@ RemoteParticipant::getLocalRTPPort()
 
 void 
 RemoteParticipant::initiateRemoteCall(const NameAddr& destination)
+{
+   SharedPtr<UserProfile> profile;
+   initiateRemoteCall(destination, profile, std::multimap<resip::Data,resip::Data>());
+}
+
+void
+RemoteParticipant::initiateRemoteCall(const NameAddr& destination, SharedPtr<UserProfile>& callingProfile, const std::multimap<resip::Data,resip::Data>& extraHeaders)
 {
    SdpContents offer;
-   SharedPtr<ConversationProfile> profile = mConversationManager.getUserAgent()->getDefaultOutgoingConversationProfile();
+   SharedPtr<UserProfile> profile = callingProfile;
+   if(!profile)
+   {
+      profile = mConversationManager.getUserAgent()->getDefaultOutgoingConversationProfile();
+   }
    buildSdpOffer(mLocalHold, offer);
    SharedPtr<SipMessage> invitemsg = mDum.makeInviteSession(
       destination, 
@@ -117,6 +143,26 @@ RemoteParticipant::initiateRemoteCall(const NameAddr& destination)
       &offer, 
       &mDialogSet);
 
+   std::multimap<resip::Data,resip::Data>::const_iterator it = extraHeaders.begin();
+   for( ; it != extraHeaders.end(); it++)
+   {
+      resip::Data headerName(it->first);
+      resip::Data value(it->second);
+      StackLog(<<"processing an extension header: " << headerName << ": " << value);
+      resip::Headers::Type hType = resip::Headers::getType(headerName.data(), (int)headerName.size());
+      if(hType == resip::Headers::UNKNOWN)
+      {
+         resip::ExtensionHeader h_Tmp(headerName.c_str());
+         resip::ParserContainer<resip::StringCategory>& pc = invitemsg->header(h_Tmp);
+         resip::StringCategory sc(value);
+         pc.push_back(sc);
+      }
+      else
+      {
+         WarningLog(<<"Discarding header '"<<headerName<<"', only extension headers permitted");
+      }
+   }
+
    mDialogSet.sendInvite(invitemsg);
 
    // Clear any pending hold/unhold requests since our offer/answer here will handle it
@@ -812,9 +858,12 @@ RemoteParticipant::buildSdpOffer(bool holdSdp, SdpContents& offer)
 {
    SdpContents::Session::Medium *audioMedium = 0;
    ConversationProfile *profile = dynamic_cast<ConversationProfile*>(mDialogSet.getUserProfile().get());
+   std::auto_ptr<SdpContents> _sessionCaps;
    if(!profile) // This can happen for UAC calls
    {
       profile = mConversationManager.getUserAgent()->getDefaultOutgoingConversationProfile().get();
+      // if using the default profile, we need a copy of the session caps that we can modify
+      _sessionCaps.reset(new SdpContents(profile->sessionCaps()));
    }
 
    // If we already have a local sdp for this sesion, then use this to form the next offer - doing so will ensure
@@ -835,6 +884,9 @@ RemoteParticipant::buildSdpOffer(bool holdSdp, SdpContents& offer)
          if(mediaIt->name() == "audio" && 
             (mediaIt->protocol() == Symbols::RTP_AVP ||
              mediaIt->protocol() == Symbols::RTP_SAVP ||
+#ifdef RTP_SAVPF_FUDGE
+             mediaIt->protocol() == Symbols::RTP_SAVPF ||
+#endif
              mediaIt->protocol() == Symbols::UDP_TLS_RTP_SAVP))
          {
             audioMedium = &(*mediaIt);
@@ -846,11 +898,15 @@ RemoteParticipant::buildSdpOffer(bool holdSdp, SdpContents& offer)
       // Add any codecs from our capabilities that may not be in current local sdp - since endpoint may have changed and may now be capable 
       // of handling codecs that it previously could not (common when endpoint is a B2BUA).
 
-      SdpContents& sessionCaps = dynamic_cast<ConversationProfile*>(mDialogSet.getUserProfile().get())->sessionCaps();
+      SdpContents* sessionCaps = _sessionCaps.get();
+      if(!sessionCaps)
+      {
+         sessionCaps = &(profile->sessionCaps());
+      }
       int highPayloadId = 96;  // Note:  static payload id's are in range of 0-96
       // Iterate through codecs in session caps and check if already in offer
-      for (std::list<SdpContents::Session::Codec>::iterator codecsIt = sessionCaps.session().media().front().codecs().begin();
-           codecsIt != sessionCaps.session().media().front().codecs().end(); codecsIt++)
+      for (std::list<SdpContents::Session::Codec>::iterator codecsIt = sessionCaps->session().media().front().codecs().begin();
+           codecsIt != sessionCaps->session().media().front().codecs().end(); codecsIt++)
       {		
          bool found=false;
          bool payloadIdCollision=false;
@@ -933,7 +989,12 @@ RemoteParticipant::buildSdpOffer(bool holdSdp, SdpContents& offer)
       }
       if(mDialogSet.getSecureMediaRequired())
       {
-         audioMedium->protocol() = Symbols::RTP_SAVP;
+#ifdef RTP_SAVPF_FUDGE
+         if(mDialogSet.peerExpectsSAVPF())
+            audioMedium->protocol() = Symbols::RTP_SAVPF;
+         else
+#endif
+            audioMedium->protocol() = Symbols::RTP_SAVP;
       }
       else
       {
@@ -1018,19 +1079,32 @@ RemoteParticipant::answerMediaLine(SdpContents::Session::Medium& mediaSessionCap
    if(sdpMediaLine.getMediaType() == sdpcontainer::SdpMediaLine::MEDIA_TYPE_AUDIO && 
       (protocolType == sdpcontainer::SdpMediaLine::PROTOCOL_TYPE_RTP_AVP ||
        protocolType == sdpcontainer::SdpMediaLine::PROTOCOL_TYPE_RTP_SAVP ||
+#ifdef RTP_SAVPF_FUDGE
+       protocolType == sdpcontainer::SdpMediaLine::PROTOCOL_TYPE_RTP_SAVPF ||
+#endif
        protocolType == sdpcontainer::SdpMediaLine::PROTOCOL_TYPE_UDP_TLS_RTP_SAVP) && 
       sdpMediaLine.getConnections().size() != 0 &&
       sdpMediaLine.getConnections().front().getPort() != 0)
    {
       SdpContents::Session::Medium medium("audio", getLocalRTPPort(), 1, 
+#ifndef RTP_SAVPF_FUDGE
                                           protocolType == sdpcontainer::SdpMediaLine::PROTOCOL_TYPE_RTP_SAVP ? Symbols::RTP_SAVP :
+#else
+                                          ( protocolType == sdpcontainer::SdpMediaLine::PROTOCOL_TYPE_RTP_SAVP
+                                           || protocolType == sdpcontainer::SdpMediaLine::PROTOCOL_TYPE_RTP_SAVPF)
+                                          ? (protocolType == sdpcontainer::SdpMediaLine::PROTOCOL_TYPE_RTP_SAVPF ? Symbols::RTP_SAVPF : Symbols::RTP_SAVP) :
+#endif
                                           (protocolType == sdpcontainer::SdpMediaLine::PROTOCOL_TYPE_UDP_TLS_RTP_SAVP ? Symbols::UDP_TLS_RTP_SAVP :
                                            Symbols::RTP_AVP));
 
       // Check secure media properties and requirements
       bool secureMediaRequired = mDialogSet.getSecureMediaRequired() || protocolType != sdpcontainer::SdpMediaLine::PROTOCOL_TYPE_RTP_AVP;
+      mDialogSet.setPeerExpectsSAVPF(protocolType == sdpcontainer::SdpMediaLine::PROTOCOL_TYPE_RTP_SAVPF);
 
       if(mDialogSet.getSecureMediaMode() == ConversationProfile::Srtp || 
+#ifdef RTP_SAVPF_FUDGE
+         protocolType == sdpcontainer::SdpMediaLine::PROTOCOL_TYPE_RTP_SAVPF ||
+#endif
          protocolType == sdpcontainer::SdpMediaLine::PROTOCOL_TYPE_RTP_SAVP)  // allow accepting of SAVP profiles, even if SRTP is not enabled as a SecureMedia mode
       {
          bool supportedCryptoSuite = false;
@@ -1212,7 +1286,12 @@ RemoteParticipant::buildSdpAnswer(const SdpContents& offer, SdpContents& answer)
    try
    {
       // copy over session capabilities
-      answer = dynamic_cast<ConversationProfile*>(mDialogSet.getUserProfile().get())->sessionCaps();
+      ConversationProfile *profile = dynamic_cast<ConversationProfile*>(mDialogSet.getUserProfile().get());
+      if(!profile)
+      {
+         profile = mConversationManager.getUserAgent()->getDefaultOutgoingConversationProfile().get();
+      }
+      answer = profile->sessionCaps();
 
       // Set sessionid and version for this answer
       UInt64 currentTime = Timer::getTimeMicroSec();
@@ -1222,7 +1301,7 @@ RemoteParticipant::buildSdpAnswer(const SdpContents& offer, SdpContents& answer)
       // Set local port in answer
       // for now we only allow 1 audio media
       assert(answer.session().media().size() == 1);
-      SdpContents::Session::Medium& mediaSessionCaps = dynamic_cast<ConversationProfile*>(mDialogSet.getUserProfile().get())->sessionCaps().session().media().front();
+      SdpContents::Session::Medium& mediaSessionCaps = profile->sessionCaps().session().media().front();
       assert(mediaSessionCaps.name() == "audio");
       assert(mediaSessionCaps.codecs().size() > 0);
 
@@ -1545,6 +1624,9 @@ RemoteParticipant::adjustRTPStreams(bool sendingOffer)
       if((*itMediaLine)->getMediaType() == sdpcontainer::SdpMediaLine::MEDIA_TYPE_AUDIO && 
          ((*itMediaLine)->getTransportProtocolType() == sdpcontainer::SdpMediaLine::PROTOCOL_TYPE_RTP_AVP ||
           (*itMediaLine)->getTransportProtocolType() == sdpcontainer::SdpMediaLine::PROTOCOL_TYPE_RTP_SAVP ||
+#ifdef RTP_SAVPF_FUDGE
+          (*itMediaLine)->getTransportProtocolType() == sdpcontainer::SdpMediaLine::PROTOCOL_TYPE_RTP_SAVPF ||
+#endif
           (*itMediaLine)->getTransportProtocolType() == sdpcontainer::SdpMediaLine::PROTOCOL_TYPE_UDP_TLS_RTP_SAVP) && 
          (*itMediaLine)->getConnections().size() != 0 &&
          (*itMediaLine)->getConnections().front().getPort() != 0)
@@ -1567,6 +1649,9 @@ RemoteParticipant::adjustRTPStreams(bool sendingOffer)
          if((*itRemMediaLine)->getMediaType() == sdpcontainer::SdpMediaLine::MEDIA_TYPE_AUDIO && 
             ((*itRemMediaLine)->getTransportProtocolType() == sdpcontainer::SdpMediaLine::PROTOCOL_TYPE_RTP_AVP ||
              (*itRemMediaLine)->getTransportProtocolType() == sdpcontainer::SdpMediaLine::PROTOCOL_TYPE_RTP_SAVP ||
+#ifdef RTP_SAVPF_FUDGE
+             (*itRemMediaLine)->getTransportProtocolType() == sdpcontainer::SdpMediaLine::PROTOCOL_TYPE_RTP_SAVPF ||
+#endif
              (*itRemMediaLine)->getTransportProtocolType() == sdpcontainer::SdpMediaLine::PROTOCOL_TYPE_UDP_TLS_RTP_SAVP) && 
             (*itRemMediaLine)->getConnections().size() != 0 &&
             (*itRemMediaLine)->getConnections().front().getPort() != 0)
@@ -1581,6 +1666,9 @@ RemoteParticipant::adjustRTPStreams(bool sendingOffer)
             // Process Crypto settings (if required) - createSRTPSession using remote key
             // Note:  Top crypto in remote sdp will always be the correct suite/key
             if(mDialogSet.getSecureMediaMode() == ConversationProfile::Srtp || 
+#ifdef RTP_SAVPF_FUDGE
+               (*itRemMediaLine)->getTransportProtocolType() == sdpcontainer::SdpMediaLine::PROTOCOL_TYPE_RTP_SAVPF ||
+#endif
                (*itRemMediaLine)->getTransportProtocolType() == sdpcontainer::SdpMediaLine::PROTOCOL_TYPE_RTP_SAVP)
             {
                DebugLog(<<"adjustRTPStreams: handle=" << mHandle << ", process crypto settings");
@@ -1927,18 +2015,25 @@ RemoteParticipant::onNewSession(ServerInviteSessionHandle h, InviteSession::Offe
    // Check for Auto-Answer indication - support draft-ietf-answer-mode-01 
    // and Answer-After parameter of Call-Info header
    ConversationProfile* profile = dynamic_cast<ConversationProfile*>(h->getUserProfile().get());
-   assert(profile);
-   bool autoAnswerRequired;
-   bool autoAnswer = profile->shouldAutoAnswer(msg, &autoAnswerRequired);
-   if(!autoAnswer && autoAnswerRequired)  // If we can't autoAnswer but it was required, we must reject the call
+   bool autoAnswer = false;
+   if(profile)
    {
-      WarningCategory warning;
-      warning.hostname() = DnsUtil::getLocalHostName();
-      warning.code() = 399; /* Misc. */
-      warning.text() = "automatic answer forbidden";
-      setHandle(0); // Don't generate any callbacks for this rejected invite
-      h->reject(403 /* Forbidden */, &warning);
-      return;
+      bool autoAnswerRequired;
+      autoAnswer = profile->shouldAutoAnswer(msg, &autoAnswerRequired);
+      if(!autoAnswer && autoAnswerRequired)  // If we can't autoAnswer but it was required, we must reject the call
+      {
+         WarningCategory warning;
+         warning.hostname() = DnsUtil::getLocalHostName();
+         warning.code() = 399; /* Misc. */
+         warning.text() = "automatic answer forbidden";
+         setHandle(0); // Don't generate any callbacks for this rejected invite
+         h->reject(403 /* Forbidden */, &warning);
+         return;
+      }
+   }
+   else
+   {
+      WarningLog(<<"bypassing logic for Auto-Answer");
    }
   
    // notify of new participant
@@ -2201,10 +2296,29 @@ RemoteParticipant::onRemoteSdpChanged(InviteSessionHandle h, const SipMessage& m
 }
 
 void
-RemoteParticipant::onInfo(InviteSessionHandle, const SipMessage& msg)
+RemoteParticipant::onInfo(InviteSessionHandle session, const SipMessage& msg)
 {
    InfoLog(<< "onInfo: handle=" << mHandle << ", " << msg.brief());
-   //assert(0);
+   if(mHandle)
+   {
+      DtmfPayloadContents* contents = dynamic_cast<DtmfPayloadContents*>(msg.getContents());
+      if(contents)
+      {
+         DtmfPayloadContents::DtmfPayload& payload = contents->dtmfPayload();
+         mConversationManager.onDtmfEvent(mHandle, payload.getEventCode(), payload.getDuration(), true);
+         session->acceptNIT();
+      }
+      else
+      {
+         WarningLog(<<"INFO message without dtmf-relay payload, rejecting");
+         session->rejectNIT();
+      }
+   }
+   else
+   {
+      WarningLog(<<"INFO message received, but mHandle not set, rejecting");
+      session->rejectNIT();
+   }
 }
 
 void
