@@ -37,6 +37,12 @@ Connection::Connection(Transport* transport,const Tuple& who, Socket socket,
    mWho.mFlowKey=(FlowKey)socket;
    InfoLog (<< "Connection::Connection: new connection created to who: " << mWho);
 
+   if(transport && isWebSocket(transport->transport()))
+   {
+      mSendingTransmissionFormat = WebSocketHandshake;
+      mReceivingTransmissionFormat = WebSocketHandshake;
+   }
+
    if(mWho.mFlowKey && ConnectionBase::transport())
    {
       getConnectionManager().addConnection(this);
@@ -118,7 +124,65 @@ Connection::performWrite()
          mSendingTransmissionFormat = Uncompressed;
       }
    }
+   else if(mSendingTransmissionFormat == WebSocketHandshake)
+   {
+      mSendingTransmissionFormat = WebSocketData;
+   }
+   else if(mSendingTransmissionFormat == WebSocketData)
+   {
+      SendData *dataWs, *oldSd;
+      const Data& dataRaw = mOutstandingSends.front()->data;
+      UInt64 dataSize = 1 + 1 + dataRaw.size();
+      UInt64 lSize = (UInt64)dataRaw.size();
+      UInt8* uBuffer;
 
+      if(lSize > 0x7D && lSize <= 0xFFFF)
+      {
+         dataSize += 2;
+      }
+      else if(lSize > 0xFFFF)
+      {
+         dataSize += 8;
+      }
+
+      oldSd = mOutstandingSends.front();
+      dataWs = new SendData(oldSd->destination,
+            Data(Data::Take, new char[(int)dataSize], (Data::size_type)dataSize),
+            oldSd->transactionId,
+            oldSd->sigcompId,
+            false);
+      assert(dataWs && dataWs->data.data());
+      uBuffer = (UInt8*)dataWs->data.data();
+
+      uBuffer[0] = 0x82;
+      if(lSize <= 0x7D){
+         uBuffer[1] = (UInt8)lSize;
+         uBuffer = &uBuffer[2];
+      }
+      else if(lSize <= 0xFFFF){
+         uBuffer[1] = 0x7E;
+         uBuffer[2] = (UInt8)((lSize >> 8) & 0xFF);
+         uBuffer[3] = (UInt8)(lSize & 0xFF);
+         uBuffer = &uBuffer[4];
+      }
+      else{
+         uBuffer[1] = 0x7F;
+         uBuffer[2] = (UInt8)((lSize >> 56) & 0xFF);
+         uBuffer[3] = (UInt8)((lSize >> 48) & 0xFF);
+         uBuffer[4] = (UInt8)((lSize >> 40) & 0xFF);
+         uBuffer[5] = (UInt8)((lSize >> 32) & 0xFF);
+         uBuffer[6] = (UInt8)((lSize >> 24) & 0xFF);
+         uBuffer[7] = (UInt8)((lSize >> 16) & 0xFF);
+         uBuffer[8] = (UInt8)((lSize >> 8) & 0xFF);
+         uBuffer[9] = (UInt8)(lSize & 0xFF);
+         uBuffer = &uBuffer[10];
+      }
+
+      memcpy(uBuffer, dataRaw.data(), dataRaw.size());
+      mOutstandingSends.front() = dataWs;
+      dataWs = 0;
+      delete oldSd;
+   }
 
 #ifdef USE_SIGCOMP
    // Perform compression here, if appropriate
@@ -162,16 +226,15 @@ Connection::performWrite()
 
    if (nBytes < 0)
    {
-      if (errno != EAGAIN && errno != EWOULDBLOCK) // Treat EGAIN and EWOULDBLOCK as the same: http://stackoverflow.com/questions/7003234/which-systems-define-eagain-and-ewouldblock-as-different-values
-      {
-         //fail(data.transactionId);
-         InfoLog(<< "Write failed on socket: " << this->getSocket() << ", closing connection");
-         return -1;
-      }
-      else
-      {
-         return 0;
-      }
+      //fail(data.transactionId);
+      InfoLog(<< "Write failed on socket: " << this->getSocket() << ", closing connection");
+      return -1;
+   }
+   else if (nBytes == 0)
+   {
+       // Nothing was written - likely socket buffers are backed up and EWOULDBLOCK was returned
+       // no need to do calculations in else statement
+       return 0;
    }
    else
    {
@@ -277,12 +340,38 @@ Connection::read()
    else
 #endif
    {
-     if(!preparseNewBytes(bytesRead))
-     {
-        // Iffy; only way we have right now to indicate that this connection has
-        // gone away.
-        bytesRead=-1;
-     }
+      if (mReceivingTransmissionFormat == WebSocketHandshake)
+      {
+         bool dropConnection = false;
+         if(wsProcessHandshake(bytesRead, dropConnection))
+         {
+            ensureWritable();
+            if(performWrites())
+            {
+               mReceivingTransmissionFormat = WebSocketData;
+            }
+         }
+         else if(dropConnection)
+         {
+            bytesRead=-1;
+         }
+      }
+      else
+      {
+         if (mReceivingTransmissionFormat == WebSocketData)
+         {
+            if(!wsProcessData(bytesRead))
+            {
+               bytesRead=-1;
+            }
+         }
+         else if(!preparseNewBytes(bytesRead))
+         {
+            // Iffy; only way we have right now to indicate that this connection has
+            // gone away.
+            bytesRead=-1;
+         }
+      }
    }
    return bytesRead;
 }
