@@ -33,6 +33,7 @@ using namespace resip;
 #define RESIPROCATE_SUBSYSTEM resip::Subsystem::DNS
 
 DnsInterface::DnsInterface(DnsStub& dnsStub) : 
+   mUdpOnlyOnNumeric(false),
    mDnsStub(dnsStub)
 {
 
@@ -49,6 +50,8 @@ static Data UdpNAPTRType("SIP+D2U");
 static Data TcpNAPTRType("SIP+D2T");
 static Data TlsNAPTRType("SIPS+D2T");
 static Data DtlsNAPTRType("SIPS+D2U");
+static Data WsNAPTRType("SIP+D2W");
+static Data WssNAPTRType("SIPS+D2W");
 void 
 DnsInterface::addTransportType(TransportType type, IpVersion version)
 {
@@ -66,7 +69,13 @@ DnsInterface::addTransportType(TransportType type, IpVersion version)
          break;
       case DTLS:
          mSupportedNaptrs.insert(DtlsNAPTRType);
-         break;         
+         break;
+      case WS:
+         mSupportedNaptrs.insert(WsNAPTRType);
+         break;
+      case WSS:
+         mSupportedNaptrs.insert(WssNAPTRType);
+         break;
       default:
          assert(0);
          break;
