@@ -44,7 +44,7 @@ UdpTransport::UdpTransport(Fifo<TransactionMessage>& fifo,
    mPollEventCnt = 0;
    mTxTryCnt = mTxMsgCnt = mTxFailCnt = 0;
    mRxTryCnt = mRxMsgCnt = mRxKeepaliveCnt = mRxTransactionCnt = 0;
-   mTuple.setType(transport());
+   mTuple.setType(UDP);
    mFd = InternalTransport::socket(transport(), version);
    mTuple.mFlowKey=(FlowKey)mFd;
    bind();      // also makes it non-blocking
@@ -316,14 +316,18 @@ UdpTransport::processRxAll()
       Tuple sender(mTuple);
       int len = processRxRecv(buffer, sender);
       if ( len <= 0 )
+      {
          break;
+      }
       ++mRxMsgCnt;
       if ( processRxParse(buffer, len, sender) )
       {
          buffer = NULL;
       }
-      if ( (mTransportFlags & RESIP_TRANSPORT_FLAG_RXALL)==0 )
+      if ( (mTransportFlags & RESIP_TRANSPORT_FLAG_RXALL) == 0 )
+      {
          break;
+      }
    }
    if ( buffer && (mTransportFlags & RESIP_TRANSPORT_FLAG_KEEP_BUFFER)!=0 )
    {
@@ -332,7 +336,9 @@ UdpTransport::processRxAll()
       buffer = NULL;
    }
    if ( buffer )
+   {
       delete[] buffer;
+   }
 }
 
 /*
@@ -359,7 +365,8 @@ UdpTransport::processRxRecv(char*& buffer, Tuple& sender)
       buffer = MsgHeaderScanner::allocateBuffer(MaxBufferSize);
    }
 
-   for (;;) {
+   for (;;) 
+   {
       // !jf! how do we tell if it discarded bytes
       // !ah! we use the len-1 trick :-(
       socklen_t slen = sender.length();
@@ -442,12 +449,12 @@ UdpTransport::processRxParse(char *buffer, int len, Tuple& sender)
          else if(resp.hasMappedAddress)
          {
 #if defined(WIN32)
-         sin_addr.S_un.S_addr = htonl(resp.mappedAddress.ipv4.addr);
+            sin_addr.S_un.S_addr = htonl(resp.mappedAddress.ipv4.addr);
 #else
-         sin_addr.s_addr = htonl(resp.mappedAddress.ipv4.addr);
+            sin_addr.s_addr = htonl(resp.mappedAddress.ipv4.addr);
 #endif
-         mStunMappedAddress = Tuple(sin_addr,resp.mappedAddress.ipv4.port, UDP);
-         mStunSuccess = true;
+            mStunMappedAddress = Tuple(sin_addr,resp.mappedAddress.ipv4.port, UDP);
+            mStunSuccess = true;
          }
       }
       return false;
@@ -512,40 +519,36 @@ UdpTransport::processRxParse(char *buffer, int len, Tuple& sender)
    // Attempt to decode SigComp message, if appropriate.
    if ((buffer[0] & 0xf8) == 0xf8)
    {
-     if (!mCompression.isEnabled())
-     {
-       InfoLog(<< "Discarding unexpected SigComp Message");
-       return false;
-     }
+      if (!mCompression.isEnabled())
+      {
+         InfoLog(<< "Discarding unexpected SigComp Message");
+         return false;
+      }
 #ifdef USE_SIGCOMP
-     char* newBuffer = MsgHeaderScanner::allocateBuffer(MaxBufferSize);
-     size_t uncompressedLength =
-       mSigcompStack->uncompressMessage(buffer, len,
-                                        newBuffer, MaxBufferSize, sc);
+      char* newBuffer = MsgHeaderScanner::allocateBuffer(MaxBufferSize);
+      size_t uncompressedLength = mSigcompStack->uncompressMessage(buffer, len, newBuffer, MaxBufferSize, sc);
 
-    DebugLog (<< "Uncompressed message from "
-              << len << " bytes to "
-              << uncompressedLength << " bytes");
+      DebugLog (<< "Uncompressed message from "
+               << len << " bytes to "
+               << uncompressedLength << " bytes");
 
+      osc::SigcompMessage *nack = mSigcompStack->getNack();
 
-     osc::SigcompMessage *nack = mSigcompStack->getNack();
+      if (nack)
+      {
+         mTxFifo.add(new SendData(tuple,
+                                  Data(nack->getDatagramMessage(),
+                                       nack->getDatagramLength()),
+                                  Data::Empty,
+                                  Data::Empty,
+                                  true));
+         delete nack;
+      }
 
-     if (nack)
-     {
-       mTxFifo.add(new SendData(tuple,
-                                Data(nack->getDatagramMessage(),
-                                     nack->getDatagramLength()),
-                                Data::Empty,
-                                Data::Empty,
-                                true)
-                  );
-       delete nack;
-     }
-
-     // delete[] buffer; NO: let caller do this if needed
-     origBufferConsumed = false;
-     buffer = newBuffer;
-     len = uncompressedLength;
+      // delete[] buffer; NO: let caller do this if needed
+      origBufferConsumed = false;
+      buffer = newBuffer;
+      len = uncompressedLength;
 #endif
    }
 
@@ -633,9 +636,9 @@ UdpTransport::processRxParse(char *buffer, int len, Tuple& sender)
       {
          send(tryLater);
       }
-     delete message; // dropping message due to congestion
-     message = 0;
-     return origBufferConsumed;
+      delete message; // dropping message due to congestion
+      message = 0;
+      return origBufferConsumed;
    }
 
    if (!basicCheck(*message))
@@ -651,45 +654,42 @@ UdpTransport::processRxParse(char *buffer, int len, Tuple& sender)
 #ifdef USE_SIGCOMP
    if (mCompression.isEnabled() && sc)
    {
-     const Via &via = message->header(h_Vias).front();
-     if (message->isRequest())
-     {
-       // For requests, the compartment ID is read out of the
-       // top via header field; if not present, we use the
-       // TCP connection for identification purposes.
-       if (via.exists(p_sigcompId))
-       {
-         Data compId = via.param(p_sigcompId);
+      const Via &via = message->header(h_Vias).front();
+      if (message->isRequest())
+      {
+         // For requests, the compartment ID is read out of the
+         // top via header field; if not present, we use the
+         // TCP connection for identification purposes.
+         if (via.exists(p_sigcompId))
+         {
+            Data compId = via.param(p_sigcompId);
+            if(!compId.empty())
+            {
+               // .bwc. Crash was happening here. Why was there an empty sigcomp id?
+               mSigcompStack->provideCompartmentId(sc, compId.data(), compId.size());
+            }
+         }
+         else
+         {
+            mSigcompStack->provideCompartmentId(sc, this, sizeof(this));
+         }
+      }
+      else
+      {
+         // For responses, the compartment ID is supposed to be
+         // the same as the compartment ID of the request. We
+         // *could* dig down into the transaction layer to try to
+         // figure this out, but that's a royal pain, and a rather
+         // severe layer violation. In practice, we're going to ferret
+         // the ID out of the the Via header field, which is where we
+         // squirreled it away when we sent this request in the first place.
+         // !bwc! This probably shouldn't be going out over the wire.
+         Data compId = via.param(p_branch).getSigcompCompartment();
          if(!compId.empty())
          {
-            // .bwc. Crash was happening here. Why was there an empty sigcomp
-            // id?
-            mSigcompStack->provideCompartmentId(
-                             sc, compId.data(), compId.size());
+            mSigcompStack->provideCompartmentId(sc, compId.data(), compId.size());
          }
-       }
-       else
-       {
-         mSigcompStack->provideCompartmentId(sc, this, sizeof(this));
-       }
-     }
-     else
-     {
-       // For responses, the compartment ID is supposed to be
-       // the same as the compartment ID of the request. We
-       // *could* dig down into the transaction layer to try to
-       // figure this out, but that's a royal pain, and a rather
-       // severe layer violation. In practice, we're going to ferret
-       // the ID out of the the Via header field, which is where we
-       // squirreled it away when we sent this request in the first place.
-       // !bwc! This probably shouldn't be going out over the wire.
-       Data compId = via.param(p_branch).getSigcompCompartment();
-       if(!compId.empty())
-       {
-         mSigcompStack->provideCompartmentId(sc, compId.data(), compId.size());
-       }
-     }
-
+      }
    }
 #endif
 
