@@ -413,6 +413,7 @@ Log::timestamp(Data& res)
    GetLocalTime(&systemTime);
    tv.tv_usec = systemTime.wMilliseconds * 1000; 
 #else 
+   struct tm localTimeResult;
    struct timeval tv; 
    int result = gettimeofday (&tv, NULL);
 #endif   
@@ -434,7 +435,11 @@ Log::timestamp(Data& res)
                 datebufSize,
                 "%Y%m%d-%H%M%S", /* guaranteed to fit in 256 chars,
                                     hence don't check return code */
-                localtime (&timeInSeconds));
+#ifdef WIN32
+                localtime (&timeInSeconds));  // Thread safe call on Windows
+#else
+                localtime_r (&timeInSeconds, &localTimeResult));  // Thread safe version of localtime on linux
+#endif
    }
    
    char msbuf[5];
@@ -607,6 +612,14 @@ Log::reset()
    getLoggerData().reset();
 }
 
+#ifndef WIN32
+void
+Log::droppingPrivileges(uid_t uid, pid_t pid)
+{
+   getLoggerData().droppingPrivileges(uid, pid);
+}
+#endif
+
 bool
 Log::isLogging(Log::Level level, const resip::Subsystem& sub)
 {
@@ -819,8 +832,7 @@ Log::ThreadData::Instance(unsigned int bytesToWrite)
                remove(oldLogFileName.c_str());
                rename(logFileName.c_str(), oldLogFileName.c_str());
             }
-            // Append to log if we have a line count or byte count limit - otherwise truncate
-            mLogger = new std::ofstream(logFileName.c_str(), std::ios_base::out | ((maxLineCount() > 0 || maxByteCount() > 0) ? std::ios_base::app : std::ios_base::trunc));
+            mLogger = new std::ofstream(logFileName.c_str(), std::ios_base::out | std::ios_base::app);
             mLineCount = 0;
          }
          mLineCount++;
@@ -838,6 +850,22 @@ Log::ThreadData::reset()
    mLogger = NULL;
 }
 
+#ifndef WIN32
+void
+Log::ThreadData::droppingPrivileges(uid_t uid, pid_t pid)
+{
+   if(mType == Log::File)
+   {
+      Data logFileName(mLogFileName != "" ? mLogFileName : "resiprocate.log");
+      if(chown(logFileName.c_str(), uid, pid) < 0)
+      {
+         // Some error occurred
+         std::cerr << "ERROR: chown failed on " << logFileName << std::endl;
+      }
+   }
+}
+#endif
+
 /* ====================================================================
  * The Vovida Software License, Version 1.0 
  * 
