@@ -131,13 +131,13 @@ WinCompat::getVersion()
 WinCompat* WinCompat::mInstance = 0;
 
 
+static Mutex WinComaptInstanceMutex;
 WinCompat *
 WinCompat::instance()
 {
-   static Mutex mutex;
    if (!mInstance)
    {
-      Lock lock(mutex);
+      Lock lock(WinComaptInstanceMutex);
       if (!mInstance)
       {
          mInstance = new WinCompat();
@@ -173,11 +173,13 @@ WinCompat::WinCompat() :
    getBestInterfaceEx = (GetBestInterfaceExProc) GetProcAddress(hLib, TEXT("GetBestInterfaceEx"));
    getAdaptersAddresses = (GetAdaptersAddressesProc) GetProcAddress(hLib, TEXT("GetAdaptersAddresses"));
    getAdaptersInfo = (GetAdaptersInfoProc) GetProcAddress(hLib, TEXT("GetAdaptersInfo"));
+   getBestRoute = (GetBestRouteProc) GetProcAddress(hLib, TEXT("GetBestRoute"));
+   getIpAddrTable = (GetIpAddrTableProc) GetProcAddress(hLib, TEXT("GetIpAddrTable"));
    if (getAdaptersAddresses == NULL || getBestInterfaceEx == NULL)
    {   
       loadLibraryWithIPv6Failed = true;
    }
-   if (getAdaptersInfo == NULL)
+   if (getAdaptersInfo == NULL || getBestRoute == NULL || getIpAddrTable == NULL)
    {
       loadLibraryWithIPv4Failed = true;
    }
@@ -185,6 +187,7 @@ WinCompat::WinCompat() :
    loadLibraryWithIPv6Failed = true;
    loadLibraryWithIPv4Failed = true;
 #endif
+   DebugLog(<< "WinCompat constructor complete!");
 }
 
 void WinCompat::destroyInstance()
@@ -243,16 +246,12 @@ WinCompat::determineSourceInterfaceWithIPv6(const GenericIPAddress& destination)
    int i;
    for (i = 0, AI = pAdapterAddresses; AI != NULL; AI = AI->Next, i++) 
    {
-      PIP_ADAPTER_UNICAST_ADDRESS defaultAdaptorAddress = 0;
       for (PIP_ADAPTER_UNICAST_ADDRESS unicast = AI->FirstUnicastAddress;
            unicast; unicast = unicast->Next)
       {
          if (unicast->Address.lpSockaddr->sa_family != family)
             continue;
 
-         // Store first address of matching family as the Adaptors default address
-         if(!defaultAdaptorAddress) defaultAdaptorAddress = unicast;
-
          if (family == AF_INET && 
              reinterpret_cast<const struct sockaddr_in*>(unicast->Address.lpSockaddr)->sin_addr.S_un.S_addr == 
                                                          destination.v4Address.sin_addr.S_un.S_addr)
@@ -260,9 +259,18 @@ WinCompat::determineSourceInterfaceWithIPv6(const GenericIPAddress& destination)
             // Return default address for NIC.  Note:  We could also just return the destination, 
             // however returning the default address for NIC is beneficial in cases where the
             // co-located registrar supports redundancy via a Virtual IP address.
-            GenericIPAddress ipaddress(*defaultAdaptorAddress->Address.lpSockaddr);
-            LocalFree(pAdapterAddresses);
-            return(ipaddress);
+            for (PIP_ADAPTER_UNICAST_ADDRESS unicastRet = AI->FirstUnicastAddress;
+                 unicastRet; unicastRet = unicastRet->Next)
+            {
+#if defined(AVOID_TRANSIENT_SOURCE_ADDRESSES)
+               if (unicastRet->Flags & IP_ADAPTER_ADDRESS_TRANSIENT)
+                  continue;
+#endif
+
+               GenericIPAddress ipaddress(*unicastRet->Address.lpSockaddr);
+               LocalFree(pAdapterAddresses);
+               return(ipaddress);
+            }
          }
 #ifdef USE_IPV6
          else if (family == AF_INET6 && 
@@ -270,16 +278,25 @@ WinCompat::determineSourceInterfaceWithIPv6(const GenericIPAddress& destination)
                          &(reinterpret_cast<const struct sockaddr_in6*>(&destination.address)->sin6_addr), 
                          sizeof(IN6_ADDR)) == 0)
          {
-            // explicitly cast to sockaddr_in6, to use that version of GenericIPAddress' ctor. If we don't, then compiler
-            // defaults to ctor for sockaddr_in (at least under Win32), which will truncate the lower-bits of the IPv6 address.
-            const struct sockaddr_in6* psa = reinterpret_cast<const struct sockaddr_in6*>(defaultAdaptorAddress->Address.lpSockaddr);
+            for (PIP_ADAPTER_UNICAST_ADDRESS unicastRet = AI->FirstUnicastAddress;
+                 unicastRet; unicastRet = unicastRet->Next)
+            {
+#if defined(AVOID_TRANSIENT_SOURCE_ADDRESSES)
+               if (unicastRet->Flags & IP_ADAPTER_ADDRESS_TRANSIENT)
+                  continue;
+#endif
 
-            // Return default address for NIC.  Note:  We could also just return the destination, 
-            // however returning the default address for NIC is beneficial in cases where the
-            // co-located registrar supports redundancy via a Virtual IP address.
-            GenericIPAddress ipaddress(*psa);
-            LocalFree(pAdapterAddresses);
-            return(ipaddress);
+               // explicitly cast to sockaddr_in6, to use that version of GenericIPAddress' ctor. If we don't, then compiler
+               // defaults to ctor for sockaddr_in (at least under Win32), which will truncate the lower-bits of the IPv6 address.
+               const struct sockaddr_in6* psa = reinterpret_cast<const struct sockaddr_in6*>(unicastRet->Address.lpSockaddr);
+
+               // Return default address for NIC.  Note:  We could also just return the destination, 
+               // however returning the default address for NIC is beneficial in cases where the
+               // co-located registrar supports redundancy via a Virtual IP address.
+               GenericIPAddress ipaddress(*psa);
+               LocalFree(pAdapterAddresses);
+               return(ipaddress);
+            }
          }
 #endif
       } 
@@ -301,6 +318,12 @@ WinCompat::determineSourceInterfaceWithIPv6(const GenericIPAddress& destination)
        {
           if (unicast->Address.lpSockaddr->sa_family != saddr->sa_family)
              continue;
+
+#if defined(AVOID_TRANSIENT_SOURCE_ADDRESSES)
+         if (unicast->Flags & IP_ADAPTER_ADDRESS_TRANSIENT)
+            continue;
+#endif
+
           if (saddr->sa_family == AF_INET && AI->IfIndex == dwBestIfIndex)
           {
              GenericIPAddress ipaddress(*unicast->Address.lpSockaddr);
@@ -339,25 +362,31 @@ WinCompat::determineSourceInterfaceWithoutIPv6(const GenericIPAddress& destinati
    memset(&sourceIP, 0, sizeof(sockaddr_in));
    sourceIP.sin_family = AF_INET;
 
-   // look throught the local ip address - first we want to see if the address is local, if 
+   // look through the local ip address - first we want to see if the address is local, if 
    // not then we want to look for the Best route
    PMIB_IPADDRTABLE  pIpAddrTable = NULL;
    ULONG addrSize = 0;
          
    // allocate the space
-   if (ERROR_INSUFFICIENT_BUFFER == GetIpAddrTable(NULL, &addrSize, FALSE))
+   DWORD ret = instance()->getIpAddrTable(NULL, &addrSize, FALSE);
+   if(ERROR_INSUFFICIENT_BUFFER == ret)
    {
       pIpAddrTable = (PMIB_IPADDRTABLE) new char [addrSize];
+      if(pIpAddrTable == 0)
+      {
+          throw Exception("Can't find source address for destination - unable to new memory", __FILE__,__LINE__);
+      }        
    } 
    else 
    {
-      throw Exception("Can't find source address for destination", __FILE__,__LINE__);
+      throw Exception("Can't find source address for destination (GetIpAddrTable to get buffer space failed), ret=" + Data((UInt32)ret), __FILE__,__LINE__);
    }
-              
-   if (NO_ERROR != GetIpAddrTable(pIpAddrTable, &addrSize, FALSE)) 
+     
+   ret = instance()->getIpAddrTable(pIpAddrTable, &addrSize, FALSE);
+   if (NO_ERROR != ret) 
    {
-       delete [] (char *) pIpAddrTable;
-       return GenericIPAddress(sourceIP);
+      delete [] (char *) pIpAddrTable;
+      throw Exception("Can't find source address for destination (GetIpAddrTable failed), addrSize=" + Data((UInt32)addrSize) + ", ret=" + Data((UInt32)ret), __FILE__,__LINE__);
    }
 
    // Check if address is local or not
@@ -375,8 +404,14 @@ WinCompat::determineSourceInterfaceWithoutIPv6(const GenericIPAddress& destinati
          DWORD dwNicIndex = pIpAddrTable->table[i].dwIndex;
          for(DWORD j = 0; j <pIpAddrTable->dwNumEntries; j++)
          {
+#if defined(AVOID_TRANSIENT_SOURCE_ADDRESSES)
+            if (pIpAddrTable->table[j].wType & MIB_IPADDR_TRANSIENT)
+               continue;
+#endif
+
             if(pIpAddrTable->table[j].dwIndex == dwNicIndex)  // Default address is first address found for NIC
             {
+               DebugLog(<< "Routing to a local address - returning default address for NIC");
                sourceIP.sin_addr.s_addr = pIpAddrTable->table[j].dwAddr;               
                delete [] (char *) pIpAddrTable;
                return GenericIPAddress(sourceIP);
@@ -389,13 +424,14 @@ WinCompat::determineSourceInterfaceWithoutIPv6(const GenericIPAddress& destinati
    MIB_IPFORWARDROW bestRoute;
    memset(&bestRoute, 0, sizeof(bestRoute));
    const sockaddr_in& sin = (const sockaddr_in&)destination.address;
-   if (NO_ERROR != GetBestRoute(sin.sin_addr.s_addr, 0, &bestRoute)) 
+   ret = instance()->getBestRoute(sin.sin_addr.s_addr, 0, &bestRoute);
+   if (NO_ERROR != ret) 
    {
       delete [] (char *) pIpAddrTable;
-      throw Exception("Can't find source address for destination", __FILE__,__LINE__);
+      throw Exception("Can't find source address for destination, ret=" + Data((UInt32)ret), __FILE__,__LINE__);
    }
       
-   // look throught the local ip address to find one that match the best route.
+   // look through the local ip address to find one that match the best route.
    enum ENICEntryPreference {ENICUnknown = 0, ENextHopNotWithinNICSubnet, ENICSubnetIsAll1s, ENICServicesNextHop};
    ENICEntryPreference eCurrSelection = ENICUnknown;
 
@@ -406,6 +442,12 @@ WinCompat::determineSourceInterfaceWithoutIPv6(const GenericIPAddress& destinati
       
       ULONG addr = pIpAddrTable->table[i].dwAddr;
       ULONG gw = bestRoute.dwForwardNextHop;
+
+#if defined(AVOID_TRANSIENT_SOURCE_ADDRESSES)
+      if (entry.wType & MIB_IPADDR_TRANSIENT)
+         continue;
+#endif
+
       if(entry.dwIndex == bestRoute.dwForwardIfIndex)    // Note: there MAY be > 1 entry with the same index, see AddIPAddress.
       {
          if( (entry.dwAddr & entry.dwMask) == (bestRoute.dwForwardNextHop & entry.dwMask) )
@@ -449,6 +491,7 @@ WinCompat::determineSourceInterfaceWithoutIPv6(const GenericIPAddress& destinati
          nicMask.s_addr = entry.dwMask;
          DebugLog(<<"IP Table entry " <<i+1 <<'/' <<pIpAddrTable->dwNumEntries <<" if-index=" <<entry.dwIndex
                   <<" NIC IP=" <<DnsUtil::inet_ntop(nicIP)
+                  << (entry.wType & MIB_IPADDR_TRANSIENT ? " (Transient)" : "")
                   <<" NIC Mask=" <<DnsUtil::inet_ntop(nicMask) );
       }
    }
