@@ -9,6 +9,7 @@
 
 #include "rutil/dns/AresDns.hxx"
 #include "rutil/GenericIPAddress.hxx"
+#include "rutil/Timer.hxx"
 
 #include "AresCompat.hxx"
 #if !defined(USE_CARES)
@@ -55,7 +56,10 @@ class AresDnsPollItem : public FdPollItemBase
    virtual void	processPollEvent(FdPollEventMask mask);
    void resetPollGrp(FdPollGrp *grp)
    {
-      mPollGrp->delPollItem(mPollHandle);
+      if(mPollGrp)
+      {
+         mPollGrp->delPollItem(mPollHandle);
+      }
       mPollHandle = 0;
       mPollGrp = grp;
       if(mPollGrp)
@@ -355,10 +359,10 @@ AresDns::internalInit(const std::vector<GenericIPAddress>& additionalNameservers
       for (int i = 0; i < (*channel)->nservers; ++i)
       {
 #ifdef USE_IPV6
-         if((*channel)->servers[i].family == AF_INET6) 
+         if((*channel)->servers[i].family == AF_INET6)
          {
             InfoLog(<< " name server: " << DnsUtil::inet_ntop((*channel)->servers[i].addr6));
-         } 
+         }
          else
 #endif
          {
@@ -443,11 +447,23 @@ bool AresDns::checkDnsChange()
          // Compare them one-by-one
          for (int i = 0; i < mChannel->nservers; ++i)
          {
-            if (mChannel->servers[i].addr.s_addr
-                != channel->servers[i].addr.s_addr)
+#ifdef USE_IPV6
+            if(channel->servers[i].family == AF_INET6)
             {
-               bRet = true;
-               break;
+               if(memcmp(&mChannel->servers[i].addr6, &channel->servers[i].addr6, sizeof(mChannel->servers[i].addr6)) != 0)
+               {
+                  bRet = true;
+                  break;
+               }
+            }
+            else
+#endif
+            {
+               if (mChannel->servers[i].addr.s_addr != channel->servers[i].addr.s_addr)
+               {
+                  bRet = true;
+                  break;
+               }
             }
          }
       }
@@ -587,6 +603,9 @@ unsigned int
 AresDns::getTimeTillNextProcessMS()
 {
    struct timeval tv;
+   unsigned maxSystemTime = resip::Timer::getMaxSystemTimeWaitMs();
+   tv.tv_sec = maxSystemTime / 1000;
+   tv.tv_usec = 1000 * (maxSystemTime % 1000);
    ares_timeout(mChannel, NULL, &tv);
    return tv.tv_sec*1000 + tv.tv_usec / 1000;
 }
@@ -614,7 +633,7 @@ AresDns::processTimers()
 #endif
 }
 
-void 
+void
 AresDns::process(FdSet& fdset)
 {
    process(fdset.read, fdset.write);
